
from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    IntegrationPackInstance,
    IntegrationPackInstanceBulkRequest,
    IntegrationPackInstanceBulkResponse,
    IntegrationPackInstanceQueryConfig,
    IntegrationPackInstanceQueryResponse,
)


class IntegrationPackInstanceService(BaseService):

    @cast_models
    def create_integration_pack_instance(
        self, request_body: IntegrationPackInstance = None
    ) -> Union[IntegrationPackInstance, str]:
        """Installs an instance of the integration pack with a specific ID in the requesting account. You can set the integrationPackOverrideName field only if you configure the specified integration pack to allow multiple installs.

        :param request_body: The request body., defaults to None
        :type request_body: IntegrationPackInstance, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackInstance, str]
        """

        Validator(IntegrationPackInstance).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackInstance",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackInstance._unmap(response)
        if content == "application/xml":
            return IntegrationPackInstance._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_integration_pack_instance(
        self, id_: str
    ) -> Union[IntegrationPackInstance, str]:
        """Retrieves the properties of the integration pack instance having the specified ID.

         The ordinary GET operation retrieves the properties of the integration pack instance having the specified ID. The bulk GET operation retrieves the properties of the integration pack instances having the specified IDs, to a maximum of 100. You can obtain integration pack instance IDs from the QUERY operation.

        :param id_: The integration pack instance ID.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackInstance, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackInstance/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackInstance._unmap(response)
        if content == "application/xml":
            return IntegrationPackInstance._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_integration_pack_instance(self, id_: str) -> None:
        """Uninstalls the integration pack instance having a specified ID from the requesting account. You can obtain this ID from a QUERY operation.

        :param id_: The integration pack instance ID.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackInstance/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_integration_pack_instance(
        self, request_body: IntegrationPackInstanceBulkRequest = None
    ) -> Union[IntegrationPackInstanceBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: IntegrationPackInstanceBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackInstanceBulkResponse, str]
        """

        Validator(IntegrationPackInstanceBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackInstance/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackInstanceBulkResponse._unmap(response)
        if content == "application/xml":
            return IntegrationPackInstanceBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_integration_pack_instance(
        self, request_body: IntegrationPackInstanceQueryConfig = None
    ) -> Union[IntegrationPackInstanceQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: IntegrationPackInstanceQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackInstanceQueryResponse, str]
        """

        Validator(IntegrationPackInstanceQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackInstance/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackInstanceQueryResponse._unmap(response)
        if content == "application/xml":
            return IntegrationPackInstanceQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_integration_pack_instance(
        self, request_body: str
    ) -> Union[IntegrationPackInstanceQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackInstanceQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackInstance/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackInstanceQueryResponse._unmap(response)
        if content == "application/xml":
            return IntegrationPackInstanceQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .oftp_connection_settings import OftpConnectionSettings
from .oftp_get_options import OftpGetOptions
from .oftp_send_options import OftpSendOptions
from .oftp_listen_options import OftpListenOptions
from .shared_communication_channel import SharedCommunicationChannel


class OftpCommunicationOptionsCommunicationSetting(Enum):
    """An enumeration representing different categories.

    :cvar DEFAULT: "default"
    :vartype DEFAULT: str
    :cvar CUSTOM: "custom"
    :vartype CUSTOM: str
    :cvar COMPONENT: "component"
    :vartype COMPONENT: str
    """

    DEFAULT = "default"
    CUSTOM = "custom"
    COMPONENT = "component"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OftpCommunicationOptionsCommunicationSetting._member_map_.values(),
            )
        )


@JsonMap(
    {
        "communication_setting": "CommunicationSetting",
        "oftp_connection_settings": "OFTPConnectionSettings",
        "oftp_get_options": "OFTPGetOptions",
        "oftp_send_options": "OFTPSendOptions",
        "oftp_server_listen_options": "OFTPServerListenOptions",
        "shared_communication_channel": "SharedCommunicationChannel",
    }
)
class OftpCommunicationOptions(BaseModel):
    """OftpCommunicationOptions

    :param communication_setting: communication_setting, defaults to None
    :type communication_setting: OftpCommunicationOptionsCommunicationSetting, optional
    :param oftp_connection_settings: oftp_connection_settings, defaults to None
    :type oftp_connection_settings: OftpConnectionSettings, optional
    :param oftp_get_options: oftp_get_options, defaults to None
    :type oftp_get_options: OftpGetOptions, optional
    :param oftp_send_options: oftp_send_options, defaults to None
    :type oftp_send_options: OftpSendOptions, optional
    :param oftp_server_listen_options: oftp_server_listen_options, defaults to None
    :type oftp_server_listen_options: OftpListenOptions, optional
    :param shared_communication_channel: shared_communication_channel, defaults to None
    :type shared_communication_channel: SharedCommunicationChannel, optional
    """

    def __init__(
        self,
        communication_setting: OftpCommunicationOptionsCommunicationSetting = SENTINEL,
        oftp_connection_settings: OftpConnectionSettings = SENTINEL,
        oftp_get_options: OftpGetOptions = SENTINEL,
        oftp_send_options: OftpSendOptions = SENTINEL,
        oftp_server_listen_options: OftpListenOptions = SENTINEL,
        shared_communication_channel: SharedCommunicationChannel = SENTINEL,
        **kwargs,
    ):
        """OftpCommunicationOptions

        :param communication_setting: communication_setting, defaults to None
        :type communication_setting: OftpCommunicationOptionsCommunicationSetting, optional
        :param oftp_connection_settings: oftp_connection_settings, defaults to None
        :type oftp_connection_settings: OftpConnectionSettings, optional
        :param oftp_get_options: oftp_get_options, defaults to None
        :type oftp_get_options: OftpGetOptions, optional
        :param oftp_send_options: oftp_send_options, defaults to None
        :type oftp_send_options: OftpSendOptions, optional
        :param oftp_server_listen_options: oftp_server_listen_options, defaults to None
        :type oftp_server_listen_options: OftpListenOptions, optional
        :param shared_communication_channel: shared_communication_channel, defaults to None
        :type shared_communication_channel: SharedCommunicationChannel, optional
        """
        if communication_setting is not SENTINEL:
            self.communication_setting = self._enum_matching(
                communication_setting,
                OftpCommunicationOptionsCommunicationSetting.list(),
                "communication_setting",
            )
        if oftp_connection_settings is not SENTINEL:
            self.oftp_connection_settings = self._define_object(
                oftp_connection_settings, OftpConnectionSettings
            )
        if oftp_get_options is not SENTINEL:
            self.oftp_get_options = self._define_object(
                oftp_get_options, OftpGetOptions
            )
        if oftp_send_options is not SENTINEL:
            self.oftp_send_options = self._define_object(
                oftp_send_options, OftpSendOptions
            )
        if oftp_server_listen_options is not SENTINEL:
            self.oftp_server_listen_options = self._define_object(
                oftp_server_listen_options, OftpListenOptions
            )
        if shared_communication_channel is not SENTINEL:
            self.shared_communication_channel = self._define_object(
                shared_communication_channel, SharedCommunicationChannel
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..execution_count_account_group import ExecutionCountAccountGroupService
from ...models import (
    ExecutionCountAccountGroupQueryResponse,
    ExecutionCountAccountGroupQueryConfig,
)


class ExecutionCountAccountGroupServiceAsync(ExecutionCountAccountGroupService):
    """
    Async Wrapper for ExecutionCountAccountGroupServiceAsync
    """

    def query_execution_count_account_group(
        self, request_body: ExecutionCountAccountGroupQueryConfig = None
    ) -> Awaitable[Union[ExecutionCountAccountGroupQueryResponse, str]]:
        return to_async(super().query_execution_count_account_group)(request_body)

    def query_more_execution_count_account_group(
        self, request_body: str
    ) -> Awaitable[Union[ExecutionCountAccountGroupQueryResponse, str]]:
        return to_async(super().query_more_execution_count_account_group)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account_expression import AccountExpression, AccountExpressionGuard
from .account_simple_expression import AccountSimpleExpression
from .account_grouping_expression import AccountGroupingExpression


@JsonMap({})
class AccountQueryConfigQueryFilter(BaseModel):
    """AccountQueryConfigQueryFilter

    :param expression: expression
    :type expression: AccountExpression
    """

    def __init__(self, expression: AccountExpression, **kwargs):
        """AccountQueryConfigQueryFilter

        :param expression: expression
        :type expression: AccountExpression
        """
        self.expression = AccountExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AccountQueryConfig(BaseModel):
    """AccountQueryConfig

    :param query_filter: query_filter
    :type query_filter: AccountQueryConfigQueryFilter
    """

    def __init__(self, query_filter: AccountQueryConfigQueryFilter, **kwargs):
        """AccountQueryConfig

        :param query_filter: query_filter
        :type query_filter: AccountQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, AccountQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .document_count_account_expression import (
    DocumentCountAccountExpression,
    DocumentCountAccountExpressionGuard,
)
from .document_count_account_simple_expression import (
    DocumentCountAccountSimpleExpression,
)
from .document_count_account_grouping_expression import (
    DocumentCountAccountGroupingExpression,
)


@JsonMap({})
class DocumentCountAccountQueryConfigQueryFilter(BaseModel):
    """DocumentCountAccountQueryConfigQueryFilter

    :param expression: expression
    :type expression: DocumentCountAccountExpression
    """

    def __init__(self, expression: DocumentCountAccountExpression, **kwargs):
        """DocumentCountAccountQueryConfigQueryFilter

        :param expression: expression
        :type expression: DocumentCountAccountExpression
        """
        self.expression = DocumentCountAccountExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class DocumentCountAccountQueryConfig(BaseModel):
    """DocumentCountAccountQueryConfig

    :param query_filter: query_filter
    :type query_filter: DocumentCountAccountQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: DocumentCountAccountQueryConfigQueryFilter, **kwargs
    ):
        """DocumentCountAccountQueryConfig

        :param query_filter: query_filter
        :type query_filter: DocumentCountAccountQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, DocumentCountAccountQueryConfigQueryFilter
        )
        self._kwargs = kwargs




import random

from typing import Generator, Optional, Tuple
from time import sleep
from .base_handler import BaseHandler
from ...transport.request import Request
from ...transport.response import Response
from ...transport.api_error import ApiError
from ...transport.request_error import RequestError


class RetryHandler(BaseHandler):
    """
    Handler for retrying requests.
    Retries the request if the previous handler in the chain returned an error or a response with a status code of 500 or higher.

    :ivar int _max_attempts: The maximum number of retry attempts.
    :ivar int _delay_in_milliseconds: The delay between retry attempts in milliseconds.
    """

    def __init__(self):
        """
        Initialize a new instance of RetryHandler.
        """
        super().__init__()
        self._max_attempts = 3
        self._delay_in_milliseconds = 150

    def handle(
        self, request: Request
    ) -> Tuple[Optional[Response], Optional[RequestError]]:
        """
        Retry the request if the response has a status code greater or equal to 500 or equal to 408 (timeout).

        :param Request request: The request to retry.
        :return: The response and any error that occurred.
        :rtype: Tuple[Optional[Response], Optional[RequestError]]
        :raises RequestError: If the handler chain is incomplete.
        """
        if self._next_handler is None:
            raise RequestError("Handler chain is incomplete")

        response, error = self._next_handler.handle(request)

        try_count = 0
        while try_count < self._max_attempts and self._should_retry(error):
            self._delay(try_count)
            response, error = self._next_handler.handle(request)
            try_count += 1

        return response, error

    def stream(
        self, request: Request
    ) -> Generator[Tuple[Optional[Response], Optional[RequestError]], None, None]:
        """
        Retry the request if the response has a status code greater or equal to 500 or equal to 408 (timeout).

        :param Request request: The request to retry.
        :return: The response and any error that occurred.
        :rtype: Generator[Tuple[Optional[Response], Optional[RequestError]], None, None]
        :raises RequestError: If the handler chain is incomplete.
        """
        if self._next_handler is None:
            raise RequestError("Handler chain is incomplete")

        try:
            try_count = 0
            stream = self._next_handler.stream(request)
            while True:
                response, error = next(stream)
                if try_count < self._max_attempts and self._should_retry(error):
                    self._delay(try_count)
                    try_count += 1
                    stream = self._next_handler.stream(request)  # Retry the request
                elif try_count >= self._max_attempts:
                    yield response, error
                    break
                else:
                    yield response, error

        except StopIteration:
            pass

    def _delay(self, try_count: int) -> None:
        jitter = random.uniform(0.5, 1.5)
        delay = self._delay_in_milliseconds * (2**try_count) * jitter / 1000
        sleep(delay)

    def _should_retry(self, error: Optional[ApiError]) -> bool:
        """
        Determine whether the request should be retried.

        :param Optional[Response] response: The response from the previous handler.
        :param Optional[ApiError] error: The error from the previous handler.
        :return: True if the request should be retried, False otherwise.
        :rtype: bool
        """
        if not error:
            return False
        return error.status == 408 or error.status >= 500




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .environment_connection_field_extension_summary_simple_expression import (
    EnvironmentConnectionFieldExtensionSummarySimpleExpression,
)
from .environment_connection_field_extension_summary_grouping_expression import (
    EnvironmentConnectionFieldExtensionSummaryGroupingExpression,
)


class EnvironmentConnectionFieldExtensionSummaryExpressionGuard(OneOfBaseModel):
    class_list = {
        "EnvironmentConnectionFieldExtensionSummarySimpleExpression": EnvironmentConnectionFieldExtensionSummarySimpleExpression,
        "EnvironmentConnectionFieldExtensionSummaryGroupingExpression": EnvironmentConnectionFieldExtensionSummaryGroupingExpression,
    }


EnvironmentConnectionFieldExtensionSummaryExpression = Union[
    EnvironmentConnectionFieldExtensionSummarySimpleExpression,
    EnvironmentConnectionFieldExtensionSummaryGroupingExpression,
]




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..trading_partner_component import TradingPartnerComponentService
from ...models import (
    TradingPartnerComponent,
    TradingPartnerComponentBulkResponse,
    TradingPartnerComponentBulkRequest,
    TradingPartnerComponentQueryResponse,
    TradingPartnerComponentQueryConfig,
)


class TradingPartnerComponentServiceAsync(TradingPartnerComponentService):
    """
    Async Wrapper for TradingPartnerComponentServiceAsync
    """

    def create_trading_partner_component(
        self, request_body: TradingPartnerComponent = None
    ) -> Awaitable[Union[TradingPartnerComponent, str]]:
        return to_async(super().create_trading_partner_component)(request_body)

    def get_trading_partner_component(
        self, id_: str
    ) -> Awaitable[Union[TradingPartnerComponent, str]]:
        return to_async(super().get_trading_partner_component)(id_)

    def update_trading_partner_component(
        self, id_: str, request_body: TradingPartnerComponent = None
    ) -> Awaitable[Union[TradingPartnerComponent, str]]:
        return to_async(super().update_trading_partner_component)(id_, request_body)

    def delete_trading_partner_component(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_trading_partner_component)(id_)

    def bulk_trading_partner_component(
        self, request_body: TradingPartnerComponentBulkRequest = None
    ) -> Awaitable[Union[TradingPartnerComponentBulkResponse, str]]:
        return to_async(super().bulk_trading_partner_component)(request_body)

    def query_trading_partner_component(
        self, request_body: TradingPartnerComponentQueryConfig = None
    ) -> Awaitable[Union[TradingPartnerComponentQueryResponse, str]]:
        return to_async(super().query_trading_partner_component)(request_body)

    def query_more_trading_partner_component(
        self, request_body: str
    ) -> Awaitable[Union[TradingPartnerComponentQueryResponse, str]]:
        return to_async(super().query_more_trading_partner_component)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "subscriber_name": "SubscriberName",
        "dead_letters_count": "deadLettersCount",
        "message_count": "messageCount",
    }
)
class TopicSubscriber(BaseModel):
    """TopicSubscriber

    :param subscriber_name: subscriber_name, defaults to None
    :type subscriber_name: str, optional
    :param dead_letters_count: dead_letters_count, defaults to None
    :type dead_letters_count: int, optional
    :param message_count: message_count, defaults to None
    :type message_count: int, optional
    """

    def __init__(
        self,
        subscriber_name: str = SENTINEL,
        dead_letters_count: int = SENTINEL,
        message_count: int = SENTINEL,
        **kwargs
    ):
        """TopicSubscriber

        :param subscriber_name: subscriber_name, defaults to None
        :type subscriber_name: str, optional
        :param dead_letters_count: dead_letters_count, defaults to None
        :type dead_letters_count: int, optional
        :param message_count: message_count, defaults to None
        :type message_count: int, optional
        """
        if subscriber_name is not SENTINEL:
            self.subscriber_name = subscriber_name
        if dead_letters_count is not SENTINEL:
            self.dead_letters_count = dead_letters_count
        if message_count is not SENTINEL:
            self.message_count = message_count
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .custom_tracked_field_expression import (
    CustomTrackedFieldExpression,
    CustomTrackedFieldExpressionGuard,
)
from .custom_tracked_field_simple_expression import CustomTrackedFieldSimpleExpression
from .custom_tracked_field_grouping_expression import (
    CustomTrackedFieldGroupingExpression,
)


@JsonMap({})
class CustomTrackedFieldQueryConfigQueryFilter(BaseModel):
    """CustomTrackedFieldQueryConfigQueryFilter

    :param expression: expression
    :type expression: CustomTrackedFieldExpression
    """

    def __init__(self, expression: CustomTrackedFieldExpression, **kwargs):
        """CustomTrackedFieldQueryConfigQueryFilter

        :param expression: expression
        :type expression: CustomTrackedFieldExpression
        """
        self.expression = CustomTrackedFieldExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class CustomTrackedFieldQueryConfig(BaseModel):
    """CustomTrackedFieldQueryConfig

    :param query_filter: query_filter
    :type query_filter: CustomTrackedFieldQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: CustomTrackedFieldQueryConfigQueryFilter, **kwargs
    ):
        """CustomTrackedFieldQueryConfig

        :param query_filter: query_filter
        :type query_filter: CustomTrackedFieldQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, CustomTrackedFieldQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CustomSystemProperties(BaseModel):
    """CustomSystemProperties

    :param property: property
    :type property: List[str]
    """

    def __init__(self, property: List[str], **kwargs):
        """CustomSystemProperties

        :param property: property
        :type property: List[str]
        """
        self.property = property
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class MapExtensionsDateTime(BaseModel):
    """MapExtensionsDateTime

    :param format: format, defaults to None
    :type format: str, optional
    """

    def __init__(self, format: str = SENTINEL, **kwargs):
        """MapExtensionsDateTime

        :param format: format, defaults to None
        :type format: str, optional
        """
        if format is not SENTINEL:
            self.format = format
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class OdetteUnhControlInfoControllingAgency(Enum):
    """An enumeration representing different categories.

    :cvar AA: "AA"
    :vartype AA: str
    :cvar AB: "AB"
    :vartype AB: str
    :cvar AC: "AC"
    :vartype AC: str
    :cvar AD: "AD"
    :vartype AD: str
    :cvar AE: "AE"
    :vartype AE: str
    :cvar CC: "CC"
    :vartype CC: str
    :cvar CE: "CE"
    :vartype CE: str
    :cvar EC: "EC"
    :vartype EC: str
    :cvar ED: "ED"
    :vartype ED: str
    :cvar EE: "EE"
    :vartype EE: str
    :cvar EN: "EN"
    :vartype EN: str
    :cvar ER: "ER"
    :vartype ER: str
    :cvar EU: "EU"
    :vartype EU: str
    :cvar EX: "EX"
    :vartype EX: str
    :cvar IA: "IA"
    :vartype IA: str
    :cvar KE: "KE"
    :vartype KE: str
    :cvar LI: "LI"
    :vartype LI: str
    :cvar OD: "OD"
    :vartype OD: str
    :cvar RI: "RI"
    :vartype RI: str
    :cvar RT: "RT"
    :vartype RT: str
    :cvar UN: "UN"
    :vartype UN: str
    """

    AA = "AA"
    AB = "AB"
    AC = "AC"
    AD = "AD"
    AE = "AE"
    CC = "CC"
    CE = "CE"
    EC = "EC"
    ED = "ED"
    EE = "EE"
    EN = "EN"
    ER = "ER"
    EU = "EU"
    EX = "EX"
    IA = "IA"
    KE = "KE"
    LI = "LI"
    OD = "OD"
    RI = "RI"
    RT = "RT"
    UN = "UN"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OdetteUnhControlInfoControllingAgency._member_map_.values(),
            )
        )


class OdetteUnhControlInfoRelease(Enum):
    """An enumeration representing different categories.

    :cvar ODETTERELEASE1: "ODETTERELEASE_1"
    :vartype ODETTERELEASE1: str
    :cvar ODETTERELEASE2: "ODETTERELEASE_2"
    :vartype ODETTERELEASE2: str
    :cvar ODETTERELEASE902: "ODETTERELEASE_902"
    :vartype ODETTERELEASE902: str
    :cvar ODETTERELEASE911: "ODETTERELEASE_911"
    :vartype ODETTERELEASE911: str
    :cvar ODETTERELEASE912: "ODETTERELEASE_912"
    :vartype ODETTERELEASE912: str
    :cvar ODETTERELEASE921: "ODETTERELEASE_921"
    :vartype ODETTERELEASE921: str
    :cvar ODETTERELEASE932: "ODETTERELEASE_932"
    :vartype ODETTERELEASE932: str
    :cvar ODETTERELEASE93A: "ODETTERELEASE_93A"
    :vartype ODETTERELEASE93A: str
    :cvar ODETTERELEASE94A: "ODETTERELEASE_94A"
    :vartype ODETTERELEASE94A: str
    :cvar ODETTERELEASE94B: "ODETTERELEASE_94B"
    :vartype ODETTERELEASE94B: str
    :cvar ODETTERELEASE95A: "ODETTERELEASE_95A"
    :vartype ODETTERELEASE95A: str
    :cvar ODETTERELEASE95B: "ODETTERELEASE_95B"
    :vartype ODETTERELEASE95B: str
    :cvar ODETTERELEASE96A: "ODETTERELEASE_96A"
    :vartype ODETTERELEASE96A: str
    :cvar ODETTERELEASE96B: "ODETTERELEASE_96B"
    :vartype ODETTERELEASE96B: str
    :cvar ODETTERELEASE97A: "ODETTERELEASE_97A"
    :vartype ODETTERELEASE97A: str
    :cvar ODETTERELEASE97B: "ODETTERELEASE_97B"
    :vartype ODETTERELEASE97B: str
    :cvar ODETTERELEASE98A: "ODETTERELEASE_98A"
    :vartype ODETTERELEASE98A: str
    :cvar ODETTERELEASE98B: "ODETTERELEASE_98B"
    :vartype ODETTERELEASE98B: str
    :cvar ODETTERELEASE99A: "ODETTERELEASE_99A"
    :vartype ODETTERELEASE99A: str
    :cvar ODETTERELEASE99B: "ODETTERELEASE_99B"
    :vartype ODETTERELEASE99B: str
    :cvar ODETTERELEASE00A: "ODETTERELEASE_00A"
    :vartype ODETTERELEASE00A: str
    :cvar ODETTERELEASE00B: "ODETTERELEASE_00B"
    :vartype ODETTERELEASE00B: str
    :cvar ODETTERELEASE01A: "ODETTERELEASE_01A"
    :vartype ODETTERELEASE01A: str
    :cvar ODETTERELEASE01B: "ODETTERELEASE_01B"
    :vartype ODETTERELEASE01B: str
    :cvar ODETTERELEASE02A: "ODETTERELEASE_02A"
    :vartype ODETTERELEASE02A: str
    :cvar ODETTERELEASE02B: "ODETTERELEASE_02B"
    :vartype ODETTERELEASE02B: str
    :cvar ODETTERELEASE03A: "ODETTERELEASE_03A"
    :vartype ODETTERELEASE03A: str
    :cvar ODETTERELEASE03B: "ODETTERELEASE_03B"
    :vartype ODETTERELEASE03B: str
    :cvar ODETTERELEASE04A: "ODETTERELEASE_04A"
    :vartype ODETTERELEASE04A: str
    :cvar ODETTERELEASE04B: "ODETTERELEASE_04B"
    :vartype ODETTERELEASE04B: str
    :cvar ODETTERELEASE05A: "ODETTERELEASE_05A"
    :vartype ODETTERELEASE05A: str
    :cvar ODETTERELEASE05B: "ODETTERELEASE_05B"
    :vartype ODETTERELEASE05B: str
    :cvar ODETTERELEASE06A: "ODETTERELEASE_06A"
    :vartype ODETTERELEASE06A: str
    :cvar ODETTERELEASE06B: "ODETTERELEASE_06B"
    :vartype ODETTERELEASE06B: str
    :cvar ODETTERELEASE07A: "ODETTERELEASE_07A"
    :vartype ODETTERELEASE07A: str
    :cvar ODETTERELEASE07B: "ODETTERELEASE_07B"
    :vartype ODETTERELEASE07B: str
    :cvar ODETTERELEASE08A: "ODETTERELEASE_08A"
    :vartype ODETTERELEASE08A: str
    :cvar ODETTERELEASE08B: "ODETTERELEASE_08B"
    :vartype ODETTERELEASE08B: str
    :cvar ODETTERELEASE09A: "ODETTERELEASE_09A"
    :vartype ODETTERELEASE09A: str
    :cvar ODETTERELEASE09B: "ODETTERELEASE_09B"
    :vartype ODETTERELEASE09B: str
    """

    ODETTERELEASE1 = "ODETTERELEASE_1"
    ODETTERELEASE2 = "ODETTERELEASE_2"
    ODETTERELEASE902 = "ODETTERELEASE_902"
    ODETTERELEASE911 = "ODETTERELEASE_911"
    ODETTERELEASE912 = "ODETTERELEASE_912"
    ODETTERELEASE921 = "ODETTERELEASE_921"
    ODETTERELEASE932 = "ODETTERELEASE_932"
    ODETTERELEASE93A = "ODETTERELEASE_93A"
    ODETTERELEASE94A = "ODETTERELEASE_94A"
    ODETTERELEASE94B = "ODETTERELEASE_94B"
    ODETTERELEASE95A = "ODETTERELEASE_95A"
    ODETTERELEASE95B = "ODETTERELEASE_95B"
    ODETTERELEASE96A = "ODETTERELEASE_96A"
    ODETTERELEASE96B = "ODETTERELEASE_96B"
    ODETTERELEASE97A = "ODETTERELEASE_97A"
    ODETTERELEASE97B = "ODETTERELEASE_97B"
    ODETTERELEASE98A = "ODETTERELEASE_98A"
    ODETTERELEASE98B = "ODETTERELEASE_98B"
    ODETTERELEASE99A = "ODETTERELEASE_99A"
    ODETTERELEASE99B = "ODETTERELEASE_99B"
    ODETTERELEASE00A = "ODETTERELEASE_00A"
    ODETTERELEASE00B = "ODETTERELEASE_00B"
    ODETTERELEASE01A = "ODETTERELEASE_01A"
    ODETTERELEASE01B = "ODETTERELEASE_01B"
    ODETTERELEASE02A = "ODETTERELEASE_02A"
    ODETTERELEASE02B = "ODETTERELEASE_02B"
    ODETTERELEASE03A = "ODETTERELEASE_03A"
    ODETTERELEASE03B = "ODETTERELEASE_03B"
    ODETTERELEASE04A = "ODETTERELEASE_04A"
    ODETTERELEASE04B = "ODETTERELEASE_04B"
    ODETTERELEASE05A = "ODETTERELEASE_05A"
    ODETTERELEASE05B = "ODETTERELEASE_05B"
    ODETTERELEASE06A = "ODETTERELEASE_06A"
    ODETTERELEASE06B = "ODETTERELEASE_06B"
    ODETTERELEASE07A = "ODETTERELEASE_07A"
    ODETTERELEASE07B = "ODETTERELEASE_07B"
    ODETTERELEASE08A = "ODETTERELEASE_08A"
    ODETTERELEASE08B = "ODETTERELEASE_08B"
    ODETTERELEASE09A = "ODETTERELEASE_09A"
    ODETTERELEASE09B = "ODETTERELEASE_09B"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, OdetteUnhControlInfoRelease._member_map_.values())
        )


class OdetteUnhControlInfoVersion(Enum):
    """An enumeration representing different categories.

    :cvar ODETTEVERSION1: "ODETTEVERSION_1"
    :vartype ODETTEVERSION1: str
    :cvar ODETTEVERSION2: "ODETTEVERSION_2"
    :vartype ODETTEVERSION2: str
    :cvar ODETTEVERSION4: "ODETTEVERSION_4"
    :vartype ODETTEVERSION4: str
    :cvar ODETTEVERSION88: "ODETTEVERSION_88"
    :vartype ODETTEVERSION88: str
    :cvar ODETTEVERSION89: "ODETTEVERSION_89"
    :vartype ODETTEVERSION89: str
    :cvar ODETTEVERSION90: "ODETTEVERSION_90"
    :vartype ODETTEVERSION90: str
    :cvar ODETTEVERSIOND: "ODETTEVERSION_D"
    :vartype ODETTEVERSIOND: str
    :cvar ODETTEVERSIONS: "ODETTEVERSION_S"
    :vartype ODETTEVERSIONS: str
    """

    ODETTEVERSION1 = "ODETTEVERSION_1"
    ODETTEVERSION2 = "ODETTEVERSION_2"
    ODETTEVERSION4 = "ODETTEVERSION_4"
    ODETTEVERSION88 = "ODETTEVERSION_88"
    ODETTEVERSION89 = "ODETTEVERSION_89"
    ODETTEVERSION90 = "ODETTEVERSION_90"
    ODETTEVERSIOND = "ODETTEVERSION_D"
    ODETTEVERSIONS = "ODETTEVERSION_S"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, OdetteUnhControlInfoVersion._member_map_.values())
        )


@JsonMap(
    {
        "assoc_assigned_code": "assocAssignedCode",
        "common_access_ref": "commonAccessRef",
        "controlling_agency": "controllingAgency",
    }
)
class OdetteUnhControlInfo(BaseModel):
    """OdetteUnhControlInfo

    :param assoc_assigned_code: assoc_assigned_code, defaults to None
    :type assoc_assigned_code: str, optional
    :param common_access_ref: common_access_ref, defaults to None
    :type common_access_ref: str, optional
    :param controlling_agency: controlling_agency, defaults to None
    :type controlling_agency: OdetteUnhControlInfoControllingAgency, optional
    :param release: release, defaults to None
    :type release: OdetteUnhControlInfoRelease, optional
    :param version: version, defaults to None
    :type version: OdetteUnhControlInfoVersion, optional
    """

    def __init__(
        self,
        assoc_assigned_code: str = SENTINEL,
        common_access_ref: str = SENTINEL,
        controlling_agency: OdetteUnhControlInfoControllingAgency = SENTINEL,
        release: OdetteUnhControlInfoRelease = SENTINEL,
        version: OdetteUnhControlInfoVersion = SENTINEL,
        **kwargs
    ):
        """OdetteUnhControlInfo

        :param assoc_assigned_code: assoc_assigned_code, defaults to None
        :type assoc_assigned_code: str, optional
        :param common_access_ref: common_access_ref, defaults to None
        :type common_access_ref: str, optional
        :param controlling_agency: controlling_agency, defaults to None
        :type controlling_agency: OdetteUnhControlInfoControllingAgency, optional
        :param release: release, defaults to None
        :type release: OdetteUnhControlInfoRelease, optional
        :param version: version, defaults to None
        :type version: OdetteUnhControlInfoVersion, optional
        """
        if assoc_assigned_code is not SENTINEL:
            self.assoc_assigned_code = assoc_assigned_code
        if common_access_ref is not SENTINEL:
            self.common_access_ref = common_access_ref
        if controlling_agency is not SENTINEL:
            self.controlling_agency = self._enum_matching(
                controlling_agency,
                OdetteUnhControlInfoControllingAgency.list(),
                "controlling_agency",
            )
        if release is not SENTINEL:
            self.release = self._enum_matching(
                release, OdetteUnhControlInfoRelease.list(), "release"
            )
        if version is not SENTINEL:
            self.version = self._enum_matching(
                version, OdetteUnhControlInfoVersion.list(), "version"
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_connection_field_extension_summary import (
    EnvironmentConnectionFieldExtensionSummaryService,
)
from ...models import (
    EnvironmentConnectionFieldExtensionSummaryQueryResponse,
    EnvironmentConnectionFieldExtensionSummaryQueryConfig,
)


class EnvironmentConnectionFieldExtensionSummaryServiceAsync(
    EnvironmentConnectionFieldExtensionSummaryService
):
    """
    Async Wrapper for EnvironmentConnectionFieldExtensionSummaryServiceAsync
    """

    def query_environment_connection_field_extension_summary(
        self, request_body: EnvironmentConnectionFieldExtensionSummaryQueryConfig = None
    ) -> Awaitable[Union[EnvironmentConnectionFieldExtensionSummaryQueryResponse, str]]:
        return to_async(super().query_environment_connection_field_extension_summary)(
            request_body
        )

    def query_more_environment_connection_field_extension_summary(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentConnectionFieldExtensionSummaryQueryResponse, str]]:
        return to_async(
            super().query_more_environment_connection_field_extension_summary
        )(request_body)




from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "account_id": "accountId",
        "assertion_encryption": "assertionEncryption",
        "authn_context": "authnContext",
        "authn_context_comparison": "authnContextComparison",
        "cert_info": "certInfo",
        "fed_id_from_name_id": "fedIdFromNameId",
        "idp_url": "idpUrl",
        "name_id_policy": "nameIdPolicy",
        "signout_redirect_url": "signoutRedirectUrl",
    }
)
class AccountSsoConfig(BaseModel):
    """AccountSsoConfig

    :param account_id: The account ID., defaults to None
    :type account_id: str, optional
    :param assertion_encryption: assertion_encryption, defaults to None
    :type assertion_encryption: bool, optional
    :param authn_context: PPT - \(Default\) Password Protected Transport, requires a username and password for authentication\<br /\>UNSPECIFIED - Accepts any type of authentication, defaults to None
    :type authn_context: str, optional
    :param authn_context_comparison: EXACT - The resulting authentication context in the authentication statement must be the exact match to at least one of the specified authentication contexts.\<br /\>MINUMUM - The resulting authentication context in the authentication statement must be at least as strong \(as deemed by the responder\) as one of the specified authentication contexts., defaults to None
    :type authn_context_comparison: str, optional
    :param cert_info: Metadata for the public certificate of the identity provider., defaults to None
    :type cert_info: str, optional
    :param certificate: Base64-encoded certificate bytes for the identity provider., defaults to None
    :type certificate: List[str], optional
    :param enabled: *true* — Enables single sign-on for the account. \<br /\>   *false* — Disables single sign-on for the account., defaults to None
    :type enabled: bool, optional
    :param fed_id_from_name_id: *true* — The federation ID is in the NameID element of the Subject element in the SAML Response document.\<br /\> *false*— The federation ID is in the FEDERATION\_ID Attribute element in the SAML Response document., defaults to None
    :type fed_id_from_name_id: bool, optional
    :param idp_url: The URL of the identity provider's single sign-on service., defaults to None
    :type idp_url: str, optional
    :param name_id_policy: *TRANSIENT* — Indicates that the content of the element is a non-constant and temporary value that should not assume any standard meaning; the identifier confirms a user is granted access without revealing the user's actual name or identity\<br /\>*UNSPECIFIED* — Indicates that identity provider can interpret the NameID attribute; the identifier confirms a user is granted access and can reveal the user's real name or identity depending on how it is defined by identity provider.\<br /\>**Important:** Entering any value other than TRANSIENT or UNSPECIFIED for the nameIdPolicy results in an exception.\<br /\>As a service provider, does not interpret the NameID value; a user is identified by comparing the *NameID* value with the *Federation ID* value., defaults to None
    :type name_id_policy: str, optional
    :param signout_redirect_url: After signing out of the, the URL that redirects the user., defaults to None
    :type signout_redirect_url: str, optional
    """

    def __init__(
        self,
        account_id: str = SENTINEL,
        assertion_encryption: bool = SENTINEL,
        authn_context: str = SENTINEL,
        authn_context_comparison: str = SENTINEL,
        cert_info: str = SENTINEL,
        certificate: List[str] = SENTINEL,
        enabled: bool = SENTINEL,
        fed_id_from_name_id: bool = SENTINEL,
        idp_url: str = SENTINEL,
        name_id_policy: str = SENTINEL,
        signout_redirect_url: str = SENTINEL,
        **kwargs
    ):
        """AccountSsoConfig

        :param account_id: The account ID., defaults to None
        :type account_id: str, optional
        :param assertion_encryption: assertion_encryption, defaults to None
        :type assertion_encryption: bool, optional
        :param authn_context: PPT - \(Default\) Password Protected Transport, requires a username and password for authentication\<br /\>UNSPECIFIED - Accepts any type of authentication, defaults to None
        :type authn_context: str, optional
        :param authn_context_comparison: EXACT - The resulting authentication context in the authentication statement must be the exact match to at least one of the specified authentication contexts.\<br /\>MINUMUM - The resulting authentication context in the authentication statement must be at least as strong \(as deemed by the responder\) as one of the specified authentication contexts., defaults to None
        :type authn_context_comparison: str, optional
        :param cert_info: Metadata for the public certificate of the identity provider., defaults to None
        :type cert_info: str, optional
        :param certificate: Base64-encoded certificate bytes for the identity provider., defaults to None
        :type certificate: List[str], optional
        :param enabled: *true* — Enables single sign-on for the account. \<br /\>   *false* — Disables single sign-on for the account., defaults to None
        :type enabled: bool, optional
        :param fed_id_from_name_id: *true* — The federation ID is in the NameID element of the Subject element in the SAML Response document.\<br /\> *false*— The federation ID is in the FEDERATION\_ID Attribute element in the SAML Response document., defaults to None
        :type fed_id_from_name_id: bool, optional
        :param idp_url: The URL of the identity provider's single sign-on service., defaults to None
        :type idp_url: str, optional
        :param name_id_policy: *TRANSIENT* — Indicates that the content of the element is a non-constant and temporary value that should not assume any standard meaning; the identifier confirms a user is granted access without revealing the user's actual name or identity\<br /\>*UNSPECIFIED* — Indicates that identity provider can interpret the NameID attribute; the identifier confirms a user is granted access and can reveal the user's real name or identity depending on how it is defined by identity provider.\<br /\>**Important:** Entering any value other than TRANSIENT or UNSPECIFIED for the nameIdPolicy results in an exception.\<br /\>As a service provider, does not interpret the NameID value; a user is identified by comparing the *NameID* value with the *Federation ID* value., defaults to None
        :type name_id_policy: str, optional
        :param signout_redirect_url: After signing out of the, the URL that redirects the user., defaults to None
        :type signout_redirect_url: str, optional
        """
        if account_id is not SENTINEL:
            self.account_id = account_id
        if assertion_encryption is not SENTINEL:
            self.assertion_encryption = assertion_encryption
        if authn_context is not SENTINEL:
            self.authn_context = authn_context
        if authn_context_comparison is not SENTINEL:
            self.authn_context_comparison = authn_context_comparison
        if cert_info is not SENTINEL:
            self.cert_info = cert_info
        if certificate is not SENTINEL:
            self.certificate = certificate
        if enabled is not SENTINEL:
            self.enabled = enabled
        if fed_id_from_name_id is not SENTINEL:
            self.fed_id_from_name_id = fed_id_from_name_id
        if idp_url is not SENTINEL:
            self.idp_url = idp_url
        if name_id_policy is not SENTINEL:
            self.name_id_policy = name_id_policy
        if signout_redirect_url is not SENTINEL:
            self.signout_redirect_url = signout_redirect_url
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .connection import Connection


@JsonMap({})
class Connections(BaseModel):
    """Connections

    :param connection: connection, defaults to None
    :type connection: List[Connection], optional
    """

    def __init__(self, connection: List[Connection] = SENTINEL, **kwargs):
        """Connections

        :param connection: connection, defaults to None
        :type connection: List[Connection], optional
        """
        if connection is not SENTINEL:
            self.connection = self._define_list(connection, Connection)
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    ComponentDiffRequest,
    ComponentDiffRequestBulkRequest,
    ComponentDiffRequestBulkResponse,
    ComponentDiffResponseCreate,
)


class ComponentDiffRequestService(BaseService):

    @cast_models
    def create_component_diff_request(
        self, request_body: ComponentDiffRequest = None
    ) -> Union[ComponentDiffResponseCreate, str]:
        """Contains a diff visualization option to help understand the differences between component versions. For more information, refer to the Postman article [Visualize request responses using Postman Visualizer](https://learning.postman.com/docs/sending-requests/response-data/visualizer/).

        :param request_body: The request body., defaults to None
        :type request_body: ComponentDiffRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentDiffResponseCreate, str]
        """

        Validator(ComponentDiffRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentDiffRequest",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentDiffResponseCreate._unmap(response)
        if content == "application/xml":
            return ComponentDiffResponseCreate._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_component_diff_request(
        self, component_id: str
    ) -> Union[ComponentDiffRequest, str]:
        """If you use Postman to make API calls, the GET response contains a diff visualization option to help understand the differences between component versions. For more information, refer to the Postman article [Visualize request responses using Postman Visualizer](https://learning.postman.com/docs/sending-requests/response-data/visualizer/). The Postman visualization feature currently supports only JSON responses.

        :param component_id: The ID of the component for which you want to compare versions.
        :type component_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentDiffRequest, str]
        """

        Validator(str).validate(component_id)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentDiffRequest/{{componentId}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("componentId", component_id)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentDiffRequest._unmap(response)
        if content == "application/xml":
            return ComponentDiffRequest._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_component_diff_request(
        self, request_body: ComponentDiffRequestBulkRequest = None
    ) -> Union[ComponentDiffRequestBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: ComponentDiffRequestBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentDiffRequestBulkResponse, str]
        """

        Validator(ComponentDiffRequestBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentDiffRequest/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentDiffRequestBulkResponse._unmap(response)
        if content == "application/xml":
            return ComponentDiffRequestBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .doc_cache_key_inputs import DocCacheKeyInputs
from .doc_cache_profile_parameters import DocCacheProfileParameters


@JsonMap(
    {
        "inputs": "Inputs",
        "outputs": "Outputs",
        "cache_index": "cacheIndex",
        "doc_cache": "docCache",
    }
)
class MapExtensionsDocCacheLookup(BaseModel):
    """MapExtensionsDocCacheLookup

    :param inputs: inputs
    :type inputs: DocCacheKeyInputs
    :param outputs: outputs
    :type outputs: DocCacheProfileParameters
    :param cache_index: cache_index, defaults to None
    :type cache_index: int, optional
    :param doc_cache: doc_cache, defaults to None
    :type doc_cache: str, optional
    """

    def __init__(
        self,
        inputs: DocCacheKeyInputs,
        outputs: DocCacheProfileParameters,
        cache_index: int = SENTINEL,
        doc_cache: str = SENTINEL,
        **kwargs,
    ):
        """MapExtensionsDocCacheLookup

        :param inputs: inputs
        :type inputs: DocCacheKeyInputs
        :param outputs: outputs
        :type outputs: DocCacheProfileParameters
        :param cache_index: cache_index, defaults to None
        :type cache_index: int, optional
        :param doc_cache: doc_cache, defaults to None
        :type doc_cache: str, optional
        """
        self.inputs = self._define_object(inputs, DocCacheKeyInputs)
        self.outputs = self._define_object(outputs, DocCacheProfileParameters)
        if cache_index is not SENTINEL:
            self.cache_index = cache_index
        if doc_cache is not SENTINEL:
            self.doc_cache = doc_cache
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.transport.utils import parse_xml_to_dict
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    PublisherIntegrationPack,
    PublisherIntegrationPackBulkRequest,
    PublisherIntegrationPackBulkResponse,
    PublisherIntegrationPackQueryConfig,
    PublisherIntegrationPackQueryResponse,
)


class PublisherIntegrationPackService(BaseService):

    @cast_models
    def create_publisher_integration_pack(
        self, request_body: PublisherIntegrationPack = None
    ) -> Union[PublisherIntegrationPack, str]:
        """Creates a single attachment or multiple attachment integration pack.

        :param request_body: The request body., defaults to None
        :type request_body: PublisherIntegrationPack, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PublisherIntegrationPack, str]
        """

        Validator(PublisherIntegrationPack).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PublisherIntegrationPack._unmap(response)
        if content == "application/xml":
            return PublisherIntegrationPack._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_publisher_integration_pack(
        self, id_: str
    ) -> Union[PublisherIntegrationPack, str]:
        """Retrieves the details of the integration pack and packaged components.
        The standard GET operation retrieves the properties of the integration pack with a specified ID.
        The bulk GET operation retrieves the properties of the integration packs with the specified IDs to a maximum of 100.

        :param id_: A unique ID assigned by the system to the integration pack.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PublisherIntegrationPack, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PublisherIntegrationPack._unmap(response)
        if content == "application/xml":
            return PublisherIntegrationPack._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_publisher_integration_pack(
        self, id_: str, request_body: PublisherIntegrationPack = None
    ) -> Union[PublisherIntegrationPack, str]:
        """The Update operation adds or removes the packaged components from the publisher integration pack.
         It also updates the description field of single and multiple attachment integration packs and the name field only for a single attachment integration pack.

         >**Note:** When updating an integration pack, you must include all the packaged components associated with it in the request body.
         If a packaged component is not included, it will be deleted upon updating an integration pack.
         For example, include all packaged components while updating the integration pack name.

        :param request_body: The request body., defaults to None
        :type request_body: PublisherIntegrationPack, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PublisherIntegrationPack, str]
        """

        Validator(PublisherIntegrationPack).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PublisherIntegrationPack._unmap(response)
        if content == "application/xml":
            return PublisherIntegrationPack._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_publisher_integration_pack(self, id_: str) -> None:
        """Deletes the publisher integration pack having a specified ID from the requesting account.
        The deleted integration pack is automatically uninstalled from all accounts where it was installed.
        Any Runtimes or environments attached to the integration pack are also detached.

        :param id_: A unique ID assigned by the system to the integration pack.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_publisher_integration_pack(
        self, request_body: PublisherIntegrationPackBulkRequest = None
    ) -> Union[PublisherIntegrationPackBulkResponse, str, any]:
        """The bulk GET operation returns multiple objects based on the supplied account IDs, to a maximum of 100. To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: PublisherIntegrationPackBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PublisherIntegrationPackBulkResponse, str, any]
        """

        Validator(PublisherIntegrationPackBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PublisherIntegrationPackBulkResponse._unmap(response)
        if content == "application/xml":
            return PublisherIntegrationPackBulkResponse._unmap(parse_xml_to_dict(response))
        if content == "example":
            return response
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_publisher_integration_pack(
        self, request_body: PublisherIntegrationPackQueryConfig = None
    ) -> Union[PublisherIntegrationPackQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: PublisherIntegrationPackQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PublisherIntegrationPackQueryResponse, str]
        """

        Validator(PublisherIntegrationPackQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PublisherIntegrationPackQueryResponse._unmap(response)
        if content == "application/xml":
            return PublisherIntegrationPackQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_publisher_integration_pack(
        self, request_body: str
    ) -> Union[PublisherIntegrationPackQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PublisherIntegrationPackQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PublisherIntegrationPack/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PublisherIntegrationPackQueryResponse._unmap(response)
        if content == "application/xml":
            return PublisherIntegrationPackQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    EnvironmentMapExtension,
    EnvironmentMapExtensionBulkRequest,
    EnvironmentMapExtensionBulkResponse,
)


class EnvironmentMapExtensionService(BaseService):

    @cast_models
    def get_environment_map_extension(
        self, id_: str
    ) -> Union[EnvironmentMapExtension, str]:
        """Retrieves an extensible map by its Environment Map Extension object ID.

         >**Note:** Extending a source or destination profile by means of browsing an external account may require including credentials in the request. The GET operation uses the credentials from the original process for the browse connection. However, because credential reuse can be problematic when sharing processes in Integration Packs, use the EXECUTE operation instead.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtension, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtension/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtension._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtension._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_environment_map_extension(
        self, request_body: EnvironmentMapExtensionBulkRequest = None
    ) -> Union[EnvironmentMapExtensionBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentMapExtensionBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtensionBulkResponse, str]
        """

        Validator(EnvironmentMapExtensionBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtension/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtensionBulkResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtensionBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def execute_environment_map_extension(
        self, id_: str, request_body: EnvironmentMapExtension = None
    ) -> Union[EnvironmentMapExtension, str]:
        """Use the EXECUTE operation when you want to customize XML profiles by reimporting them from endpoint applications. The EXECUTE operation returns the current Environment Map Extension configuration similar to the GET operation.

         It also accepts connection credentials and automatically connects to the external application to retrieve additional custom fields for that profile. You must have the Runtime Management privilege to perform the EXECUTE operation. If you have the Runtime Management Read Access privilege, you cannot post connection credentials.

         For information about using these operations to retrieve or update map functions, refer to [Environment Map Extension functions](/docs/APIs/PlatformAPI/Environment_Map_Extension_functions).

         Include the `SourceBrowse` and `DestinationBrowse` sections as appropriate to browse the respective profile and include the required BrowseFields for the given connector. If you need to call the EXECUTE action repeatedly for the same map, you can alternatively use the `sessionId` to avoid having to supply the connector fields in subsequent calls. Session caching lasts about 30 minutes.

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentMapExtension, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtension, str]
        """

        Validator(EnvironmentMapExtension).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtension/execute/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtension._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtension._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .connector_version import ConnectorVersion


@JsonMap({"connector_version": "ConnectorVersion", "id_": "id"})
class AtomConnectorVersions(BaseModel):
    """AtomConnectorVersions

    :param connector_version: Each pair of `name` and `version` fields represents a connector listed on the **Runtime Management page \> Runtime & Connector Versions** tab., defaults to None
    :type connector_version: List[ConnectorVersion], optional
    :param id_: The ID of the Runtime, Runtime cluster, or Runtime cloud., defaults to None
    :type id_: str, optional
    """

    def __init__(
        self,
        connector_version: List[ConnectorVersion] = SENTINEL,
        id_: str = SENTINEL,
        **kwargs,
    ):
        """AtomConnectorVersions

        :param connector_version: Each pair of `name` and `version` fields represents a connector listed on the **Runtime Management page \> Runtime & Connector Versions** tab., defaults to None
        :type connector_version: List[ConnectorVersion], optional
        :param id_: The ID of the Runtime, Runtime cluster, or Runtime cloud., defaults to None
        :type id_: str, optional
        """
        if connector_version is not SENTINEL:
            self.connector_version = self._define_list(
                connector_version, ConnectorVersion
            )
        if id_ is not SENTINEL:
            self.id_ = id_
        self._kwargs = kwargs




from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"atom_id": "atomId", "node_id": "nodeId"})
class NodeOffboard(BaseModel):
    """NodeOffboard

    :param atom_id: A unique, -generated identifier assigned to the Runtime cluster or Runtime cloud. In the  user interface, this ID appears on the **Runtime Information** panel of Runtime Management., defaults to None
    :type atom_id: str, optional
    :param node_id: The ID of the Runtime cluster or Cloud node that is intended for deletion. In the  user interface, this ID appears on the **Cluster Status** panel of Runtime Management., defaults to None
    :type node_id: List[str], optional
    """

    def __init__(
        self, atom_id: str = SENTINEL, node_id: List[str] = SENTINEL, **kwargs
    ):
        """NodeOffboard

        :param atom_id: A unique, -generated identifier assigned to the Runtime cluster or Runtime cloud. In the  user interface, this ID appears on the **Runtime Information** panel of Runtime Management., defaults to None
        :type atom_id: str, optional
        :param node_id: The ID of the Runtime cluster or Cloud node that is intended for deletion. In the  user interface, this ID appears on the **Cluster Status** panel of Runtime Management., defaults to None
        :type node_id: List[str], optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if node_id is not SENTINEL:
            self.node_id = node_id
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class RosettaNetOptionsVersion(Enum):
    """An enumeration representing different categories.

    :cvar V11: "v11"
    :vartype V11: str
    :cvar V20: "v20"
    :vartype V20: str
    """

    V11 = "v11"
    V20 = "v20"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, RosettaNetOptionsVersion._member_map_.values())
        )


@JsonMap(
    {
        "filter_signals": "filterSignals",
        "outbound_document_validation": "outboundDocumentValidation",
        "reject_duplicate_transactions": "rejectDuplicateTransactions",
    }
)
class RosettaNetOptions(BaseModel):
    """RosettaNetOptions

    :param filter_signals: filter_signals, defaults to None
    :type filter_signals: bool, optional
    :param outbound_document_validation: outbound_document_validation, defaults to None
    :type outbound_document_validation: bool, optional
    :param reject_duplicate_transactions: reject_duplicate_transactions, defaults to None
    :type reject_duplicate_transactions: bool, optional
    :param version: version, defaults to None
    :type version: RosettaNetOptionsVersion, optional
    """

    def __init__(
        self,
        filter_signals: bool = SENTINEL,
        outbound_document_validation: bool = SENTINEL,
        reject_duplicate_transactions: bool = SENTINEL,
        version: RosettaNetOptionsVersion = SENTINEL,
        **kwargs
    ):
        """RosettaNetOptions

        :param filter_signals: filter_signals, defaults to None
        :type filter_signals: bool, optional
        :param outbound_document_validation: outbound_document_validation, defaults to None
        :type outbound_document_validation: bool, optional
        :param reject_duplicate_transactions: reject_duplicate_transactions, defaults to None
        :type reject_duplicate_transactions: bool, optional
        :param version: version, defaults to None
        :type version: RosettaNetOptionsVersion, optional
        """
        if filter_signals is not SENTINEL:
            self.filter_signals = filter_signals
        if outbound_document_validation is not SENTINEL:
            self.outbound_document_validation = outbound_document_validation
        if reject_duplicate_transactions is not SENTINEL:
            self.reject_duplicate_transactions = reject_duplicate_transactions
        if version is not SENTINEL:
            self.version = self._enum_matching(
                version, RosettaNetOptionsVersion.list(), "version"
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_atom_attachment import EnvironmentAtomAttachmentService
from ...models import (
    EnvironmentAtomAttachment,
    EnvironmentAtomAttachmentQueryResponse,
    EnvironmentAtomAttachmentQueryConfig,
)


class EnvironmentAtomAttachmentServiceAsync(EnvironmentAtomAttachmentService):
    """
    Async Wrapper for EnvironmentAtomAttachmentServiceAsync
    """

    def create_environment_atom_attachment(
        self, request_body: EnvironmentAtomAttachment = None
    ) -> Awaitable[Union[EnvironmentAtomAttachment, str]]:
        return to_async(super().create_environment_atom_attachment)(request_body)

    def query_environment_atom_attachment(
        self, request_body: EnvironmentAtomAttachmentQueryConfig = None
    ) -> Awaitable[Union[EnvironmentAtomAttachmentQueryResponse, str]]:
        return to_async(super().query_environment_atom_attachment)(request_body)

    def query_more_environment_atom_attachment(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentAtomAttachmentQueryResponse, str]]:
        return to_async(super().query_more_environment_atom_attachment)(request_body)

    def delete_environment_atom_attachment(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_environment_atom_attachment)(id_)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"component_id": "componentId", "released_version": "releasedVersion"})
class ReleasePackagedComponent(BaseModel):
    """ReleasePackagedComponent

    :param component_id: The ID of the component., defaults to None
    :type component_id: str, optional
    :param released_version: The released version of the packaged component., defaults to None
    :type released_version: str, optional
    :param version: Packaged component version of the component that will be included in the next release of this integration pack., defaults to None
    :type version: str, optional
    """

    def __init__(
        self,
        component_id: str = SENTINEL,
        released_version: str = SENTINEL,
        version: str = SENTINEL,
        **kwargs
    ):
        """ReleasePackagedComponent

        :param component_id: The ID of the component., defaults to None
        :type component_id: str, optional
        :param released_version: The released version of the packaged component., defaults to None
        :type released_version: str, optional
        :param version: Packaged component version of the component that will be included in the next release of this integration pack., defaults to None
        :type version: str, optional
        """
        if component_id is not SENTINEL:
            self.component_id = component_id
        if released_version is not SENTINEL:
            self.released_version = released_version
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    ComponentMetadata,
    ComponentMetadataBulkRequest,
    ComponentMetadataBulkResponse,
    ComponentMetadataQueryConfig,
    ComponentMetadataQueryResponse,
)


class ComponentMetadataService(BaseService):

    @cast_models
    def create_component_metadata(
        self, request_body: ComponentMetadata = None
    ) -> Union[ComponentMetadata, str]:
        """The ability to create a new component is not supported at this time. Although, you can create a deleted component, but you cannot create a new component. You will receive an error if you do not specify the deleted component ID in the create request.

        :param request_body: The request body., defaults to None
        :type request_body: ComponentMetadata, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentMetadata, str]
        """

        Validator(ComponentMetadata).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentMetadata._unmap(response)
        if content == "application/xml":
            return ComponentMetadata._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_component_metadata(self, id_: str) -> Union[ComponentMetadata, str]:
        """Returns the latest component revision if you do not provide the version. Providing the version in the format of `<componentId>` ~ `<version>`, returns the specific version of the component.

        :param id_: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentMetadata, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentMetadata._unmap(response)
        if content == "application/xml":
            return ComponentMetadata._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_component_metadata(
        self, id_: str, request_body: ComponentMetadata = None
    ) -> Union[ComponentMetadata, str]:
        """Only `name` and `folderId` may be updated. They are optional and will only be modified if included in the UPDATE request. `folderId` must be a valid, non-deleted folder. If `folderId` is included in the request but with a blank value, it defaults to the root folder.

        :param request_body: The request body., defaults to None
        :type request_body: ComponentMetadata, optional
        :param id_: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentMetadata, str]
        """

        Validator(ComponentMetadata).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentMetadata._unmap(response)
        if content == "application/xml":
            return ComponentMetadata._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_component_metadata(self, id_: str) -> None:
        """Lets you delete required components. Note that deleting a component does NOT delete dependent components.

        :param id_: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_component_metadata(
        self, request_body: ComponentMetadataBulkRequest = None
    ) -> Union[ComponentMetadataBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: ComponentMetadataBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentMetadataBulkResponse, str]
        """

        Validator(ComponentMetadataBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentMetadataBulkResponse._unmap(response)
        if content == "application/xml":
            return ComponentMetadataBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_component_metadata(
        self, request_body: ComponentMetadataQueryConfig = None
    ) -> Union[ComponentMetadataQueryResponse, str]:
        """- By default, QUERY results include previous revisions including deleted versions. Use query filters to exclude previous and deleted versions if desired. For more examples of querying components, see Component Metadata API example requests mentioned above in the API description.
         - The `version` field must be accompanied by the `componentId` field. You can query all other fields.
         - The `copiedFromComponentId` field must accompany the `copiedFromComponentVersion` field.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: ComponentMetadataQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentMetadataQueryResponse, str]
        """

        Validator(ComponentMetadataQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentMetadataQueryResponse._unmap(response)
        if content == "application/xml":
            return ComponentMetadataQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_component_metadata(
        self, request_body: str
    ) -> Union[ComponentMetadataQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ComponentMetadataQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ComponentMetadata/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ComponentMetadataQueryResponse._unmap(response)
        if content == "application/xml":
            return ComponentMetadataQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .throughput_account_group_expression import (
    ThroughputAccountGroupExpression,
    ThroughputAccountGroupExpressionGuard,
)
from .throughput_account_group_simple_expression import (
    ThroughputAccountGroupSimpleExpression,
)
from .throughput_account_group_grouping_expression import (
    ThroughputAccountGroupGroupingExpression,
)


@JsonMap({})
class ThroughputAccountGroupQueryConfigQueryFilter(BaseModel):
    """ThroughputAccountGroupQueryConfigQueryFilter

    :param expression: expression
    :type expression: ThroughputAccountGroupExpression
    """

    def __init__(self, expression: ThroughputAccountGroupExpression, **kwargs):
        """ThroughputAccountGroupQueryConfigQueryFilter

        :param expression: expression
        :type expression: ThroughputAccountGroupExpression
        """
        self.expression = ThroughputAccountGroupExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ThroughputAccountGroupQueryConfig(BaseModel):
    """ThroughputAccountGroupQueryConfig

    :param query_filter: query_filter
    :type query_filter: ThroughputAccountGroupQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ThroughputAccountGroupQueryConfigQueryFilter, **kwargs
    ):
        """ThroughputAccountGroupQueryConfig

        :param query_filter: query_filter
        :type query_filter: ThroughputAccountGroupQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ThroughputAccountGroupQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import Roles


class GetAssignableRolesService(BaseService):

    @cast_models
    def get_get_assignable_roles(self) -> Union[Roles, str]:
        """You can use the Get Assignable Roles operation to retrieve a list of roles that are assignable under a account.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Roles, str]
        """

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/GetAssignableRoles",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Roles._unmap(response)
        if content == "application/xml":
            return Roles._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .mllpssl_options import MllpsslOptions
from .edi_delimiter import EdiDelimiter


@JsonMap(
    {
        "mllpssl_options": "MLLPSSLOptions",
        "end_block": "endBlock",
        "end_data": "endData",
        "halt_timeout": "haltTimeout",
        "inactivity_timeout": "inactivityTimeout",
        "max_connections": "maxConnections",
        "max_retry": "maxRetry",
        "receive_timeout": "receiveTimeout",
        "send_timeout": "sendTimeout",
        "start_block": "startBlock",
    }
)
class MllpSendSettings(BaseModel):
    """MllpSendSettings

    :param mllpssl_options: mllpssl_options
    :type mllpssl_options: MllpsslOptions
    :param end_block: end_block
    :type end_block: EdiDelimiter
    :param end_data: end_data
    :type end_data: EdiDelimiter
    :param halt_timeout: halt_timeout, defaults to None
    :type halt_timeout: bool, optional
    :param host: host
    :type host: str
    :param inactivity_timeout: inactivity_timeout, defaults to None
    :type inactivity_timeout: int, optional
    :param max_connections: max_connections, defaults to None
    :type max_connections: int, optional
    :param max_retry: max_retry, defaults to None
    :type max_retry: int, optional
    :param persistent: persistent, defaults to None
    :type persistent: bool, optional
    :param port: port
    :type port: int
    :param receive_timeout: receive_timeout, defaults to None
    :type receive_timeout: int, optional
    :param send_timeout: send_timeout, defaults to None
    :type send_timeout: int, optional
    :param start_block: start_block
    :type start_block: EdiDelimiter
    """

    def __init__(
        self,
        mllpssl_options: MllpsslOptions = SENTINEL,
        end_block: EdiDelimiter = SENTINEL,
        end_data: EdiDelimiter = SENTINEL,
        host: str = SENTINEL,
        port: int = SENTINEL,
        start_block: EdiDelimiter = SENTINEL,
        halt_timeout: bool = SENTINEL,
        inactivity_timeout: int = SENTINEL,
        max_connections: int = SENTINEL,
        max_retry: int = SENTINEL,
        persistent: bool = SENTINEL,
        receive_timeout: int = SENTINEL,
        send_timeout: int = SENTINEL,
        **kwargs,
    ):
        """MllpSendSettings

        :param mllpssl_options: mllpssl_options
        :type mllpssl_options: MllpsslOptions
        :param end_block: end_block
        :type end_block: EdiDelimiter
        :param end_data: end_data
        :type end_data: EdiDelimiter
        :param halt_timeout: halt_timeout, defaults to None
        :type halt_timeout: bool, optional
        :param host: host
        :type host: str
        :param inactivity_timeout: inactivity_timeout, defaults to None
        :type inactivity_timeout: int, optional
        :param max_connections: max_connections, defaults to None
        :type max_connections: int, optional
        :param max_retry: max_retry, defaults to None
        :type max_retry: int, optional
        :param persistent: persistent, defaults to None
        :type persistent: bool, optional
        :param port: port
        :type port: int
        :param receive_timeout: receive_timeout, defaults to None
        :type receive_timeout: int, optional
        :param send_timeout: send_timeout, defaults to None
        :type send_timeout: int, optional
        :param start_block: start_block
        :type start_block: EdiDelimiter
        """
        if mllpssl_options is not SENTINEL:
            self.mllpssl_options = self._define_object(mllpssl_options, MllpsslOptions)
        if end_block is not SENTINEL:
            self.end_block = self._define_object(end_block, EdiDelimiter)
        if end_data is not SENTINEL:
            self.end_data = self._define_object(end_data, EdiDelimiter)
        if halt_timeout is not SENTINEL:
            self.halt_timeout = halt_timeout
        if host is not SENTINEL:
            self.host = host
        if inactivity_timeout is not SENTINEL:
            self.inactivity_timeout = inactivity_timeout
        if max_connections is not SENTINEL:
            self.max_connections = max_connections
        if max_retry is not SENTINEL:
            self.max_retry = max_retry
        if persistent is not SENTINEL:
            self.persistent = persistent
        if port is not SENTINEL:
            self.port = port
        if receive_timeout is not SENTINEL:
            self.receive_timeout = receive_timeout
        if send_timeout is not SENTINEL:
            self.send_timeout = send_timeout
        if start_block is not SENTINEL:
            self.start_block = self._define_object(start_block, EdiDelimiter)
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .organization_component_simple_expression import (
    OrganizationComponentSimpleExpression,
)
from .organization_component_grouping_expression import (
    OrganizationComponentGroupingExpression,
)


class OrganizationComponentExpressionGuard(OneOfBaseModel):
    class_list = {
        "OrganizationComponentSimpleExpression": OrganizationComponentSimpleExpression,
        "OrganizationComponentGroupingExpression": OrganizationComponentGroupingExpression,
    }


OrganizationComponentExpression = Union[
    OrganizationComponentSimpleExpression, OrganizationComponentGroupingExpression
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .element import Element


@JsonMap({})
class HttpReflectHeaders(BaseModel):
    """HttpReflectHeaders

    :param element: element, defaults to None
    :type element: List[Element], optional
    """

    def __init__(self, element: List[Element] = SENTINEL, **kwargs):
        """HttpReflectHeaders

        :param element: element, defaults to None
        :type element: List[Element], optional
        """
        if element is not SENTINEL:
            self.element = self._define_list(element, Element)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .standard_properties import StandardProperties
from .system_properties import SystemProperties
from .custom_runtime_properties import CustomRuntimeProperties
from .custom_system_properties import CustomSystemProperties


@JsonMap(
    {
        "partial_update": "partialUpdate",
        "runtime_id": "runtimeId",
        "standard_properties": "standardProperties",
        "system_properties": "systemProperties",
        "custom_runtime_properties": "customRuntimeProperties",
        "custom_system_properties": "customSystemProperties",
        "should_restart_runtime": "shouldRestartRuntime",
    }
)
class RuntimeProperties(BaseModel):
    """RuntimeProperties

    :param standard_properties: standard_properties
    :type standard_properties: StandardProperties
    :param system_properties: system_properties
    :type system_properties: SystemProperties
    :param custom_runtime_properties: custom_runtime_properties
    :type custom_runtime_properties: CustomRuntimeProperties
    :param custom_system_properties: custom_system_properties
    :type custom_system_properties: CustomSystemProperties
    :param partial_update: partial_update, defaults to None
    :type partial_update: bool, optional
    :param runtime_id: runtime_id, defaults to None
    :type runtime_id: str, optional
    :param should_restart_runtime: should_restart_runtime, defaults to None
    :type should_restart_runtime: bool, optional
    """

    def __init__(
        self,
        standard_properties: StandardProperties,
        system_properties: SystemProperties,
        custom_runtime_properties: CustomRuntimeProperties,
        custom_system_properties: CustomSystemProperties,
        partial_update: bool = SENTINEL,
        runtime_id: str = SENTINEL,
        should_restart_runtime: bool = SENTINEL,
        **kwargs,
    ):
        """RuntimeProperties

        :param standard_properties: standard_properties
        :type standard_properties: StandardProperties
        :param system_properties: system_properties
        :type system_properties: SystemProperties
        :param custom_runtime_properties: custom_runtime_properties
        :type custom_runtime_properties: CustomRuntimeProperties
        :param custom_system_properties: custom_system_properties
        :type custom_system_properties: CustomSystemProperties
        :param partial_update: partial_update, defaults to None
        :type partial_update: bool, optional
        :param runtime_id: runtime_id, defaults to None
        :type runtime_id: str, optional
        :param should_restart_runtime: should_restart_runtime, defaults to None
        :type should_restart_runtime: bool, optional
        """
        self.standard_properties = self._define_object(
            standard_properties, StandardProperties
        )
        self.system_properties = self._define_object(
            system_properties, SystemProperties
        )
        self.custom_runtime_properties = self._define_object(
            custom_runtime_properties, CustomRuntimeProperties
        )
        self.custom_system_properties = self._define_object(
            custom_system_properties, CustomSystemProperties
        )
        if partial_update is not SENTINEL:
            self.partial_update = partial_update
        if runtime_id is not SENTINEL:
            self.runtime_id = runtime_id
        if should_restart_runtime is not SENTINEL:
            self.should_restart_runtime = should_restart_runtime
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .account_cloud_attachment_properties_default import AccountCloudAttachmentPropertiesDefault


@JsonMap(
    {
        "number_of_results": "numberOfResults",
        "response_status_code": "responseStatusCode",
    }
)
class AccountCloudAttachmentPropertiesDefaultAsyncResponse(BaseModel):
    """AccountCloudAttachmentPropertiesDefaultAsyncResponse

    :param number_of_results: number_of_results, defaults to None
    :type number_of_results: int, optional
    :param response_status_code: response_status_code
    :type response_status_code: int
    :param result: result, defaults to None
    :type result: List[AccountCloudAttachmentPropertiesDefault], optional
    """

    def __init__(
        self,
        response_status_code: int,
        number_of_results: int = SENTINEL,
        result: List[AccountCloudAttachmentPropertiesDefault] = SENTINEL,
        **kwargs,
    ):
        """AccountCloudAttachmentPropertiesDefaultAsyncResponse

        :param number_of_results: number_of_results, defaults to None
        :type number_of_results: int, optional
        :param response_status_code: response_status_code
        :type response_status_code: int
        :param result: result, defaults to None
        :type result: List[AccountCloudAttachmentPropertiesDefault], optional
        """
        if number_of_results is not SENTINEL:
            self.number_of_results = number_of_results
        self.response_status_code = response_status_code
        if result is not SENTINEL:
            self.result = self._define_list(result, AccountCloudAttachmentPropertiesDefault)
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import JavaRollback


class JavaRollbackService(BaseService):

    @cast_models
    def execute_java_rollback(
        self, id_: str, request_body: JavaRollback = None
    ) -> Union[JavaRollback, str]:
        """Returns a Runtime, Runtime cluster, Runtime cloud, Authentication Broker, or Gateway to use the previous version of Java with an EXECUTE operation.

         - After performing the EXECUTE operation, you can determine the success of returning to an earlier version when the **Update to use <new Java version>.<minor_version>** link displays on the following pages, indicating that a more recent version is available for upgrade:

          - For Runtimes, Runtime clusters, and Runtime clouds — the **Runtime Information** panel (**Manage** > **Runtime Management** of the user interface).

          - For Brokers — the **Broker Information** panel (**Configure Server** > **Authentication** of the user interface).

          - For API Gateways — the **Gateway Information** panel (**Configure Server** > **Gateways** of the user interface).

          To verify a successful rollback on a Runtime using the user interface, you can also navigate to **Runtime Management** > **Startup Properties** and reference the Java version number listed in the **Java Home** field.

        - The container must be online to use the Rollback Java operation.

         >**Important:** Only the node that runs upgrades (typically the head node) restarts automatically to run the updated Java version for Runtimes, Runtime clusters, and Runtime clouds. You must restart all other cluster nodes to successfully return to a previous version of Java.
         > To successfully return to a previous version of Java for API Management Gateways and Authentication Brokers, you must restart all Gateways and Brokers.

        :param request_body: The request body., defaults to None
        :type request_body: JavaRollback, optional
        :param id_: The unique ID assigned by the system to the container.

        1. Use the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds found in the user interface by navigating to **Manage** > **Runtime Management** and viewing the Runtime Information panel for a selected container.

        2. Use the Gateway ID found in the user interface by navigating to **Configure Server** > **Gateways** > `<gatewayName>` > Gateway Information panel.

        3. Use the Broker ID found in the user interface by navigating to **Configure Server** > **Authentication** > `<brokerName>` > Broker Information panel.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[JavaRollback, str]
        """

        Validator(JavaRollback).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/JavaRollback/execute/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return JavaRollback._unmap(response)
        if content == "application/xml":
            return JavaRollback._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class As2WorkloadDefault(Enum):
    """An enumeration representing different categories.

    :cvar GENERAL: "GENERAL"
    :vartype GENERAL: str
    :cvar LOWLATENCYDEBUG: "LOW_LATENCY_DEBUG"
    :vartype LOWLATENCYDEBUG: str
    """

    GENERAL = "GENERAL"
    LOWLATENCYDEBUG = "LOW_LATENCY_DEBUG"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, As2WorkloadDefault._member_map_.values()))


class FlowControlParallelProcessTypeOverrideDefault(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "NONE"
    :vartype NONE: str
    :cvar THREADS: "THREADS"
    :vartype THREADS: str
    :cvar PROCESSES: "PROCESSES"
    :vartype PROCESSES: str
    """

    NONE = "NONE"
    THREADS = "THREADS"
    PROCESSES = "PROCESSES"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                FlowControlParallelProcessTypeOverrideDefault._member_map_.values(),
            )
        )


class HttpWorkloadDefault(Enum):
    """An enumeration representing different categories.

    :cvar GENERAL: "GENERAL"
    :vartype GENERAL: str
    :cvar LOWLATENCYDEBUG: "LOW_LATENCY_DEBUG"
    :vartype LOWLATENCYDEBUG: str
    :cvar LOWLATENCY: "LOW_LATENCY"
    :vartype LOWLATENCY: str
    """

    GENERAL = "GENERAL"
    LOWLATENCYDEBUG = "LOW_LATENCY_DEBUG"
    LOWLATENCY = "LOW_LATENCY"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, HttpWorkloadDefault._member_map_.values()))


@JsonMap(
    {
        "account_disk_usage": "accountDiskUsage",
        "as2_workload": "as2Workload",
        "atom_input_size": "atomInputSize",
        "atom_output_overflow_size": "atomOutputOverflowSize",
        "atom_working_overflow_size": "atomWorkingOverflowSize",
        "cloud_account_execution_limit": "cloudAccountExecutionLimit",
        "cloud_account_execution_warning_offset": "cloudAccountExecutionWarningOffset",
        "container_id": "containerId",
        "download_runnerlogs": "downloadRunnerlogs",
        "enable_account_data_archiving": "enableAccountDataArchiving",
        "enable_atom_worker_warmup": "enableAtomWorkerWarmup",
        "flow_control_parallel_process_type_override": "flowControlParallelProcessTypeOverride",
        "http_request_rate": "httpRequestRate",
        "http_workload": "httpWorkload",
        "listener_max_concurrent_executions": "listenerMaxConcurrentExecutions",
        "max_connector_track_docs": "maxConnectorTrackDocs",
        "min_numberof_atom_workers": "minNumberofAtomWorkers",
        "numberof_atom_workers": "numberofAtomWorkers",
        "queue_commit_batch_limit": "queueCommitBatchLimit",
        "queue_incoming_message_rate_limit": "queueIncomingMessageRateLimit",
        "queue_max_batch_size": "queueMaxBatchSize",
        "queue_max_doc_size": "queueMaxDocSize",
        "queue_msg_throttle_rate": "queueMsgThrottleRate",
        "queue_use_file_persistence": "queueUseFilePersistence",
        "test_mode_max_doc_bytes": "testModeMaxDocBytes",
        "test_mode_max_docs": "testModeMaxDocs",
        "worker_elastic_scaling_threshold": "workerElasticScalingThreshold",
        "worker_max_execution_time": "workerMaxExecutionTime",
        "worker_max_general_execution_time": "workerMaxGeneralExecutionTime",
        "worker_max_queued_executions": "workerMaxQueuedExecutions",
        "worker_max_running_executions": "workerMaxRunningExecutions",
        "worker_queued_execution_timeout": "workerQueuedExecutionTimeout",
    }
)
class AccountCloudAttachmentPropertiesDefault(BaseModel):
    """AccountCloudAttachmentPropertiesDefault

    :param account_disk_usage: account_disk_usage, defaults to None
    :type account_disk_usage: int, optional
    :param as2_workload: as2_workload, defaults to None
    :type as2_workload: As2WorkloadDefault, optional
    :param atom_input_size: atom_input_size, defaults to None
    :type atom_input_size: int, optional
    :param atom_output_overflow_size: atom_output_overflow_size, defaults to None
    :type atom_output_overflow_size: int, optional
    :param atom_working_overflow_size: atom_working_overflow_size, defaults to None
    :type atom_working_overflow_size: int, optional
    :param cloud_account_execution_limit: cloud_account_execution_limit, defaults to None
    :type cloud_account_execution_limit: int, optional
    :param cloud_account_execution_warning_offset: cloud_account_execution_warning_offset, defaults to None
    :type cloud_account_execution_warning_offset: int, optional
    :param container_id: container_id, defaults to None
    :type container_id: str, optional
    :param download_runnerlogs: download_runnerlogs, defaults to None
    :type download_runnerlogs: bool, optional
    :param enable_account_data_archiving: enable_account_data_archiving, defaults to None
    :type enable_account_data_archiving: bool, optional
    :param enable_atom_worker_warmup: enable_atom_worker_warmup, defaults to None
    :type enable_atom_worker_warmup: bool, optional
    :param flow_control_parallel_process_type_override: flow_control_parallel_process_type_override, defaults to None
    :type flow_control_parallel_process_type_override: FlowControlParallelProcessTypeOverrideDefault, optional
    :param http_request_rate: http_request_rate, defaults to None
    :type http_request_rate: int, optional
    :param http_workload: http_workload, defaults to None
    :type http_workload: HttpWorkloadDefault, optional
    :param listener_max_concurrent_executions: listener_max_concurrent_executions, defaults to None
    :type listener_max_concurrent_executions: int, optional
    :param max_connector_track_docs: max_connector_track_docs, defaults to None
    :type max_connector_track_docs: int, optional
    :param min_numberof_atom_workers: min_numberof_atom_workers, defaults to None
    :type min_numberof_atom_workers: int, optional
    :param numberof_atom_workers: numberof_atom_workers, defaults to None
    :type numberof_atom_workers: int, optional
    :param queue_commit_batch_limit: queue_commit_batch_limit, defaults to None
    :type queue_commit_batch_limit: int, optional
    :param queue_incoming_message_rate_limit: queue_incoming_message_rate_limit, defaults to None
    :type queue_incoming_message_rate_limit: int, optional
    :param queue_max_batch_size: queue_max_batch_size, defaults to None
    :type queue_max_batch_size: int, optional
    :param queue_max_doc_size: queue_max_doc_size, defaults to None
    :type queue_max_doc_size: int, optional
    :param queue_msg_throttle_rate: queue_msg_throttle_rate, defaults to None
    :type queue_msg_throttle_rate: int, optional
    :param queue_use_file_persistence: queue_use_file_persistence, defaults to None
    :type queue_use_file_persistence: bool, optional
    :param test_mode_max_doc_bytes: test_mode_max_doc_bytes, defaults to None
    :type test_mode_max_doc_bytes: int, optional
    :param test_mode_max_docs: test_mode_max_docs, defaults to None
    :type test_mode_max_docs: int, optional
    :param worker_elastic_scaling_threshold: worker_elastic_scaling_threshold, defaults to None
    :type worker_elastic_scaling_threshold: int, optional
    :param worker_max_execution_time: worker_max_execution_time, defaults to None
    :type worker_max_execution_time: int, optional
    :param worker_max_general_execution_time: worker_max_general_execution_time, defaults to None
    :type worker_max_general_execution_time: int, optional
    :param worker_max_queued_executions: worker_max_queued_executions, defaults to None
    :type worker_max_queued_executions: int, optional
    :param worker_max_running_executions: worker_max_running_executions, defaults to None
    :type worker_max_running_executions: int, optional
    :param worker_queued_execution_timeout: worker_queued_execution_timeout, defaults to None
    :type worker_queued_execution_timeout: int, optional
    """

    def __init__(
        self,
        account_disk_usage: int = SENTINEL,
        as2_workload: As2WorkloadDefault = SENTINEL,
        atom_input_size: int = SENTINEL,
        atom_output_overflow_size: int = SENTINEL,
        atom_working_overflow_size: int = SENTINEL,
        cloud_account_execution_limit: int = SENTINEL,
        cloud_account_execution_warning_offset: int = SENTINEL,
        container_id: str = SENTINEL,
        download_runnerlogs: bool = SENTINEL,
        enable_account_data_archiving: bool = SENTINEL,
        enable_atom_worker_warmup: bool = SENTINEL,
        flow_control_parallel_process_type_override: FlowControlParallelProcessTypeOverrideDefault = SENTINEL,
        http_request_rate: int = SENTINEL,
        http_workload: HttpWorkloadDefault = SENTINEL,
        listener_max_concurrent_executions: int = SENTINEL,
        max_connector_track_docs: int = SENTINEL,
        min_numberof_atom_workers: int = SENTINEL,
        numberof_atom_workers: int = SENTINEL,
        queue_commit_batch_limit: int = SENTINEL,
        queue_incoming_message_rate_limit: int = SENTINEL,
        queue_max_batch_size: int = SENTINEL,
        queue_max_doc_size: int = SENTINEL,
        queue_msg_throttle_rate: int = SENTINEL,
        queue_use_file_persistence: bool = SENTINEL,
        test_mode_max_doc_bytes: int = SENTINEL,
        test_mode_max_docs: int = SENTINEL,
        worker_elastic_scaling_threshold: int = SENTINEL,
        worker_max_execution_time: int = SENTINEL,
        worker_max_general_execution_time: int = SENTINEL,
        worker_max_queued_executions: int = SENTINEL,
        worker_max_running_executions: int = SENTINEL,
        worker_queued_execution_timeout: int = SENTINEL,
        **kwargs
    ):
        """AccountCloudAttachmentPropertiesDefault

        :param account_disk_usage: account_disk_usage, defaults to None
        :type account_disk_usage: int, optional
        :param as2_workload: as2_workload, defaults to None
        :type as2_workload: As2WorkloadDefault, optional
        :param atom_input_size: atom_input_size, defaults to None
        :type atom_input_size: int, optional
        :param atom_output_overflow_size: atom_output_overflow_size, defaults to None
        :type atom_output_overflow_size: int, optional
        :param atom_working_overflow_size: atom_working_overflow_size, defaults to None
        :type atom_working_overflow_size: int, optional
        :param cloud_account_execution_limit: cloud_account_execution_limit, defaults to None
        :type cloud_account_execution_limit: int, optional
        :param cloud_account_execution_warning_offset: cloud_account_execution_warning_offset, defaults to None
        :type cloud_account_execution_warning_offset: int, optional
        :param container_id: container_id, defaults to None
        :type container_id: str, optional
        :param download_runnerlogs: download_runnerlogs, defaults to None
        :type download_runnerlogs: bool, optional
        :param enable_account_data_archiving: enable_account_data_archiving, defaults to None
        :type enable_account_data_archiving: bool, optional
        :param enable_atom_worker_warmup: enable_atom_worker_warmup, defaults to None
        :type enable_atom_worker_warmup: bool, optional
        :param flow_control_parallel_process_type_override: flow_control_parallel_process_type_override, defaults to None
        :type flow_control_parallel_process_type_override: FlowControlParallelProcessTypeOverrideDefault, optional
        :param http_request_rate: http_request_rate, defaults to None
        :type http_request_rate: int, optional
        :param http_workload: http_workload, defaults to None
        :type http_workload: HttpWorkloadDefault, optional
        :param listener_max_concurrent_executions: listener_max_concurrent_executions, defaults to None
        :type listener_max_concurrent_executions: int, optional
        :param max_connector_track_docs: max_connector_track_docs, defaults to None
        :type max_connector_track_docs: int, optional
        :param min_numberof_atom_workers: min_numberof_atom_workers, defaults to None
        :type min_numberof_atom_workers: int, optional
        :param numberof_atom_workers: numberof_atom_workers, defaults to None
        :type numberof_atom_workers: int, optional
        :param queue_commit_batch_limit: queue_commit_batch_limit, defaults to None
        :type queue_commit_batch_limit: int, optional
        :param queue_incoming_message_rate_limit: queue_incoming_message_rate_limit, defaults to None
        :type queue_incoming_message_rate_limit: int, optional
        :param queue_max_batch_size: queue_max_batch_size, defaults to None
        :type queue_max_batch_size: int, optional
        :param queue_max_doc_size: queue_max_doc_size, defaults to None
        :type queue_max_doc_size: int, optional
        :param queue_msg_throttle_rate: queue_msg_throttle_rate, defaults to None
        :type queue_msg_throttle_rate: int, optional
        :param queue_use_file_persistence: queue_use_file_persistence, defaults to None
        :type queue_use_file_persistence: bool, optional
        :param test_mode_max_doc_bytes: test_mode_max_doc_bytes, defaults to None
        :type test_mode_max_doc_bytes: int, optional
        :param test_mode_max_docs: test_mode_max_docs, defaults to None
        :type test_mode_max_docs: int, optional
        :param worker_elastic_scaling_threshold: worker_elastic_scaling_threshold, defaults to None
        :type worker_elastic_scaling_threshold: int, optional
        :param worker_max_execution_time: worker_max_execution_time, defaults to None
        :type worker_max_execution_time: int, optional
        :param worker_max_general_execution_time: worker_max_general_execution_time, defaults to None
        :type worker_max_general_execution_time: int, optional
        :param worker_max_queued_executions: worker_max_queued_executions, defaults to None
        :type worker_max_queued_executions: int, optional
        :param worker_max_running_executions: worker_max_running_executions, defaults to None
        :type worker_max_running_executions: int, optional
        :param worker_queued_execution_timeout: worker_queued_execution_timeout, defaults to None
        :type worker_queued_execution_timeout: int, optional
        """
        if account_disk_usage is not SENTINEL:
            self.account_disk_usage = account_disk_usage
        if as2_workload is not SENTINEL:
            self.as2_workload = self._enum_matching(
                as2_workload, As2WorkloadDefault.list(), "as2_workload"
            )
        if atom_input_size is not SENTINEL:
            self.atom_input_size = atom_input_size
        if atom_output_overflow_size is not SENTINEL:
            self.atom_output_overflow_size = atom_output_overflow_size
        if atom_working_overflow_size is not SENTINEL:
            self.atom_working_overflow_size = atom_working_overflow_size
        if cloud_account_execution_limit is not SENTINEL:
            self.cloud_account_execution_limit = cloud_account_execution_limit
        if cloud_account_execution_warning_offset is not SENTINEL:
            self.cloud_account_execution_warning_offset = (
                cloud_account_execution_warning_offset
            )
        if container_id is not SENTINEL:
            self.container_id = container_id
        if download_runnerlogs is not SENTINEL:
            self.download_runnerlogs = download_runnerlogs
        if enable_account_data_archiving is not SENTINEL:
            self.enable_account_data_archiving = enable_account_data_archiving
        if enable_atom_worker_warmup is not SENTINEL:
            self.enable_atom_worker_warmup = enable_atom_worker_warmup
        if flow_control_parallel_process_type_override is not SENTINEL:
            self.flow_control_parallel_process_type_override = self._enum_matching(
                flow_control_parallel_process_type_override,
                FlowControlParallelProcessTypeOverrideDefault.list(),
                "flow_control_parallel_process_type_override",
            )
        if http_request_rate is not SENTINEL:
            self.http_request_rate = http_request_rate
        if http_workload is not SENTINEL:
            self.http_workload = self._enum_matching(
                http_workload, HttpWorkloadDefault.list(), "http_workload"
            )
        if listener_max_concurrent_executions is not SENTINEL:
            self.listener_max_concurrent_executions = listener_max_concurrent_executions
        if max_connector_track_docs is not SENTINEL:
            self.max_connector_track_docs = max_connector_track_docs
        if min_numberof_atom_workers is not SENTINEL:
            self.min_numberof_atom_workers = min_numberof_atom_workers
        if numberof_atom_workers is not SENTINEL:
            self.numberof_atom_workers = numberof_atom_workers
        if queue_commit_batch_limit is not SENTINEL:
            self.queue_commit_batch_limit = queue_commit_batch_limit
        if queue_incoming_message_rate_limit is not SENTINEL:
            self.queue_incoming_message_rate_limit = queue_incoming_message_rate_limit
        if queue_max_batch_size is not SENTINEL:
            self.queue_max_batch_size = queue_max_batch_size
        if queue_max_doc_size is not SENTINEL:
            self.queue_max_doc_size = queue_max_doc_size
        if queue_msg_throttle_rate is not SENTINEL:
            self.queue_msg_throttle_rate = queue_msg_throttle_rate
        if queue_use_file_persistence is not SENTINEL:
            self.queue_use_file_persistence = queue_use_file_persistence
        if test_mode_max_doc_bytes is not SENTINEL:
            self.test_mode_max_doc_bytes = test_mode_max_doc_bytes
        if test_mode_max_docs is not SENTINEL:
            self.test_mode_max_docs = test_mode_max_docs
        if worker_elastic_scaling_threshold is not SENTINEL:
            self.worker_elastic_scaling_threshold = worker_elastic_scaling_threshold
        if worker_max_execution_time is not SENTINEL:
            self.worker_max_execution_time = worker_max_execution_time
        if worker_max_general_execution_time is not SENTINEL:
            self.worker_max_general_execution_time = worker_max_general_execution_time
        if worker_max_queued_executions is not SENTINEL:
            self.worker_max_queued_executions = worker_max_queued_executions
        if worker_max_running_executions is not SENTINEL:
            self.worker_max_running_executions = worker_max_running_executions
        if worker_queued_execution_timeout is not SENTINEL:
            self.worker_queued_execution_timeout = worker_queued_execution_timeout
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_role_expression import (
    EnvironmentRoleExpression,
    EnvironmentRoleExpressionGuard,
)
from .environment_role_simple_expression import EnvironmentRoleSimpleExpression
from .environment_role_grouping_expression import EnvironmentRoleGroupingExpression


@JsonMap({})
class EnvironmentRoleQueryConfigQueryFilter(BaseModel):
    """EnvironmentRoleQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentRoleExpression
    """

    def __init__(self, expression: EnvironmentRoleExpression, **kwargs):
        """EnvironmentRoleQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentRoleExpression
        """
        self.expression = EnvironmentRoleExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentRoleQueryConfig(BaseModel):
    """EnvironmentRoleQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentRoleQueryConfigQueryFilter
    """

    def __init__(self, query_filter: EnvironmentRoleQueryConfigQueryFilter, **kwargs):
        """EnvironmentRoleQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentRoleQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EnvironmentRoleQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .permitted_roles import PermittedRoles


@JsonMap(
    {
        "permitted_roles": "PermittedRoles",
        "full_path": "fullPath",
        "id_": "id",
        "parent_id": "parentId",
        "parent_name": "parentName",
    }
)
class Folder(BaseModel):
    """Folder

    :param permitted_roles: Optional. The defined role assigned to the available folder object., defaults to None
    :type permitted_roles: PermittedRoles, optional
    :param deleted: Read only. Indicates if the component is deleted. A true value indicates a deleted status, whereas a false value indicates an active status., defaults to None
    :type deleted: bool, optional
    :param full_path: Read only. The full path of the folder location in which the component most currently resides within the Component Explorer hierarchy., defaults to None
    :type full_path: str, optional
    :param id_: Required. Read only. The -generated, unique identifier of the folder., defaults to None
    :type id_: str, optional
    :param name: Required. The user-defined name given to the folder., defaults to None
    :type name: str, optional
    :param parent_id: Required. The -generated, unique identifier of the parent folder., defaults to None
    :type parent_id: str, optional
    :param parent_name: parent_name, defaults to None
    :type parent_name: str, optional
    """

    def __init__(
        self,
        permitted_roles: PermittedRoles = SENTINEL,
        deleted: bool = SENTINEL,
        full_path: str = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        parent_id: str = SENTINEL,
        parent_name: str = SENTINEL,
        **kwargs,
    ):
        """Folder

        :param permitted_roles: Optional. The defined role assigned to the available folder object., defaults to None
        :type permitted_roles: PermittedRoles, optional
        :param deleted: Read only. Indicates if the component is deleted. A true value indicates a deleted status, whereas a false value indicates an active status., defaults to None
        :type deleted: bool, optional
        :param full_path: Read only. The full path of the folder location in which the component most currently resides within the Component Explorer hierarchy., defaults to None
        :type full_path: str, optional
        :param id_: Required. Read only. The -generated, unique identifier of the folder., defaults to None
        :type id_: str, optional
        :param name: Required. The user-defined name given to the folder., defaults to None
        :type name: str, optional
        :param parent_id: Required. The -generated, unique identifier of the parent folder., defaults to None
        :type parent_id: str, optional
        :param parent_name: parent_name, defaults to None
        :type parent_name: str, optional
        """
        if permitted_roles is not SENTINEL:
            self.permitted_roles = self._define_object(permitted_roles, PermittedRoles)
        if deleted is not SENTINEL:
            self.deleted = self._define_bool("deleted", deleted, nullable=True)
        if full_path is not SENTINEL:
            self.full_path = full_path
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        if parent_id is not SENTINEL:
            self.parent_id = parent_id
        if parent_name is not SENTINEL:
            self.parent_name = parent_name
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .parameter import Parameter


@JsonMap({})
class HttpRequestParameters(BaseModel):
    """HttpRequestParameters

    :param parameter: parameter, defaults to None
    :type parameter: List[Parameter], optional
    """

    def __init__(self, parameter: List[Parameter] = SENTINEL, **kwargs):
        """HttpRequestParameters

        :param parameter: parameter, defaults to None
        :type parameter: List[Parameter], optional
        """
        if parameter is not SENTINEL:
            self.parameter = self._define_list(parameter, Parameter)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .component_atom_attachment_expression import (
    ComponentAtomAttachmentExpression,
    ComponentAtomAttachmentExpressionGuard,
)
from .component_atom_attachment_simple_expression import (
    ComponentAtomAttachmentSimpleExpression,
)
from .component_atom_attachment_grouping_expression import (
    ComponentAtomAttachmentGroupingExpression,
)


@JsonMap({})
class ComponentAtomAttachmentQueryConfigQueryFilter(BaseModel):
    """ComponentAtomAttachmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: ComponentAtomAttachmentExpression
    """

    def __init__(self, expression: ComponentAtomAttachmentExpression, **kwargs):
        """ComponentAtomAttachmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: ComponentAtomAttachmentExpression
        """
        self.expression = ComponentAtomAttachmentExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ComponentAtomAttachmentQueryConfig(BaseModel):
    """ComponentAtomAttachmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: ComponentAtomAttachmentQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ComponentAtomAttachmentQueryConfigQueryFilter, **kwargs
    ):
        """ComponentAtomAttachmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: ComponentAtomAttachmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ComponentAtomAttachmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .process_schedules_simple_expression import ProcessSchedulesSimpleExpression
from .process_schedules_grouping_expression import ProcessSchedulesGroupingExpression


class ProcessSchedulesExpressionGuard(OneOfBaseModel):
    class_list = {
        "ProcessSchedulesSimpleExpression": ProcessSchedulesSimpleExpression,
        "ProcessSchedulesGroupingExpression": ProcessSchedulesGroupingExpression,
    }


ProcessSchedulesExpression = Union[
    ProcessSchedulesSimpleExpression, ProcessSchedulesGroupingExpression
]




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .environment_map_extension_user_defined_function_summary_simple_expression import (
    EnvironmentMapExtensionUserDefinedFunctionSummarySimpleExpression,
)
from .environment_map_extension_user_defined_function_summary_grouping_expression import (
    EnvironmentMapExtensionUserDefinedFunctionSummaryGroupingExpression,
)


class EnvironmentMapExtensionUserDefinedFunctionSummaryExpressionGuard(OneOfBaseModel):
    class_list = {
        "EnvironmentMapExtensionUserDefinedFunctionSummarySimpleExpression": EnvironmentMapExtensionUserDefinedFunctionSummarySimpleExpression,
        "EnvironmentMapExtensionUserDefinedFunctionSummaryGroupingExpression": EnvironmentMapExtensionUserDefinedFunctionSummaryGroupingExpression,
    }


EnvironmentMapExtensionUserDefinedFunctionSummaryExpression = Union[
    EnvironmentMapExtensionUserDefinedFunctionSummarySimpleExpression,
    EnvironmentMapExtensionUserDefinedFunctionSummaryGroupingExpression,
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    EnvironmentMapExtensionUserDefinedFunction,
    EnvironmentMapExtensionUserDefinedFunctionBulkRequest,
    EnvironmentMapExtensionUserDefinedFunctionBulkResponse,
)


class EnvironmentMapExtensionUserDefinedFunctionService(BaseService):

    @cast_models
    def create_environment_map_extension_user_defined_function(
        self, request_body: EnvironmentMapExtensionUserDefinedFunction = None
    ) -> Union[EnvironmentMapExtensionUserDefinedFunction, str]:
        """The CREATE operation creates a new extensible user-defined function. User-defined functions created using the Environment Map Extension User Defined Function object exists only at the environment extension level and are tied to a single map extension only.

         When creating a new user-defined function, you define individual function steps that make up the greater user-defined function. Then, in the `<Mappings>` section of the request, you determine how to map or link each step to and from the function's input and output.

         >**Caution:** Creating new functions requires all existing input and output values in the request regardless if they are mapped or populated with a default value. Otherwise, it overrides and removes those variables from the function.

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentMapExtensionUserDefinedFunction, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtensionUserDefinedFunction, str]
        """

        Validator(EnvironmentMapExtensionUserDefinedFunction).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtensionUserDefinedFunction",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtensionUserDefinedFunction._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtensionUserDefinedFunction._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_environment_map_extension_user_defined_function(
        self, id_: str
    ) -> Union[EnvironmentMapExtensionUserDefinedFunction, str]:
        """Retrieves an extensible user-defined function associated with a given environment map extension function ID.

        :param id_: Represents the unique, system-generated ID of the extended user-defined function.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtensionUserDefinedFunction, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtensionUserDefinedFunction/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtensionUserDefinedFunction._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtensionUserDefinedFunction._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_environment_map_extension_user_defined_function(
        self, id_: str, request_body: EnvironmentMapExtensionUserDefinedFunction = None
    ) -> Union[EnvironmentMapExtensionUserDefinedFunction, str]:
        """Updates the extended configuration for a single user-defined function.

         >**Caution:** Updating functions require all existing input and output values in the request regardless if they are mapped or populated with a default value. Otherwise, it overrides and removes those variables from the function.

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentMapExtensionUserDefinedFunction, optional
        :param id_: Represents the unique, system-generated ID of the extended user-defined function.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtensionUserDefinedFunction, str]
        """

        Validator(EnvironmentMapExtensionUserDefinedFunction).is_optional().validate(
            request_body
        )
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtensionUserDefinedFunction/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtensionUserDefinedFunction._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtensionUserDefinedFunction._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_environment_map_extension_user_defined_function(self, id_: str) -> None:
        """Deletes the specified user-defined function. Deleted user-defined functions return a status of true and are no longer available for use in an API call or on the user interface.

         ### Restoring a deleted user-defined function

         Reinstate a deleted user-defined function by providing the function's id in a CREATE operation. You cannot make changes to a function during restoration (in other words, you cannot edit its values in a RESTORE request). By restoring a deleted function, it becomes available for use in an API call and in the user interface. After a successful RESTORE operation, the function returns a deleted status of false.

        :param id_: Represents the unique, system-generated ID of the extended user-defined function.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtensionUserDefinedFunction/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_environment_map_extension_user_defined_function(
        self, request_body: EnvironmentMapExtensionUserDefinedFunctionBulkRequest = None
    ) -> Union[EnvironmentMapExtensionUserDefinedFunctionBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentMapExtensionUserDefinedFunctionBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtensionUserDefinedFunctionBulkResponse, str]
        """

        Validator(
            EnvironmentMapExtensionUserDefinedFunctionBulkRequest
        ).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtensionUserDefinedFunction/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtensionUserDefinedFunctionBulkResponse._unmap(
                response
            )
        if content == "application/xml":
            return EnvironmentMapExtensionUserDefinedFunctionBulkResponse._unmap(
                response
            )
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..clear_queue import ClearQueueService
from ...models import ClearQueueRequest


class ClearQueueServiceAsync(ClearQueueService):
    """
    Async Wrapper for ClearQueueServiceAsync
    """

    def execute_clear_queue(
        self, id_: str, request_body: ClearQueueRequest = None
    ) -> Awaitable[Union[ClearQueueRequest, str]]:
        return to_async(super().execute_clear_queue)(id_, request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .shared_web_server_cors_origin import SharedWebServerCorsOrigin


@JsonMap({})
class SharedWebServerCors(BaseModel):
    """SharedWebServerCors

    :param origins: origins, defaults to None
    :type origins: List[SharedWebServerCorsOrigin], optional
    """

    def __init__(self, origins: List[SharedWebServerCorsOrigin] = SENTINEL, **kwargs):
        """SharedWebServerCors

        :param origins: origins, defaults to None
        :type origins: List[SharedWebServerCorsOrigin], optional
        """
        if origins is not SENTINEL:
            self.origins = self._define_list(origins, SharedWebServerCorsOrigin)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..runtime_observability_settings import RuntimeObservabilitySettingsService
from ...models import (
    RuntimeObservabilitySettings,
    RuntimeObservabilitySettingsRequest,
    AsyncOperationTokenResult,
    RuntimeObservabilitySettingsAsyncResponse,
)


class RuntimeObservabilitySettingsServiceAsync(RuntimeObservabilitySettingsService):
    """
    Async Wrapper for RuntimeObservabilitySettingsServiceAsync
    """

    def update_runtime_observability_settings(
        self, id_: str, request_body: RuntimeObservabilitySettingsRequest = None
    ) -> Awaitable[Union[RuntimeObservabilitySettings, str]]:
        return to_async(super().update_runtime_observability_settings)(
            id_, request_body
        )

    def async_get_runtime_observability_settings(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_runtime_observability_settings)(id_)

    def async_token_runtime_observability_settings(
        self, token: str
    ) -> Awaitable[Union[RuntimeObservabilitySettingsAsyncResponse, str]]:
        return to_async(super().async_token_runtime_observability_settings)(token)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..throughput_account import ThroughputAccountService
from ...models import ThroughputAccountQueryResponse, ThroughputAccountQueryConfig


class ThroughputAccountServiceAsync(ThroughputAccountService):
    """
    Async Wrapper for ThroughputAccountServiceAsync
    """

    def query_throughput_account(
        self, request_body: ThroughputAccountQueryConfig = None
    ) -> Awaitable[Union[ThroughputAccountQueryResponse, str]]:
        return to_async(super().query_throughput_account)(request_body)

    def query_more_throughput_account(
        self, request_body: str
    ) -> Awaitable[Union[ThroughputAccountQueryResponse, str]]:
        return to_async(super().query_more_throughput_account)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .async_token import AsyncToken


@JsonMap({"async_token": "asyncToken", "response_status_code": "responseStatusCode"})
class AsyncOperationTokenResult(BaseModel):
    """AsyncOperationTokenResult

    :param async_token: async_token
    :type async_token: AsyncToken
    :param response_status_code: The status code returned from a request, as follows:   - 202 — Initialized the Listener status request and is in progress (QUERY response).  - 200 — Listener status request is complete (GET response).
    :type response_status_code: int
    """

    def __init__(self, async_token: AsyncToken, response_status_code: int, **kwargs):
        """AsyncOperationTokenResult

        :param async_token: async_token
        :type async_token: AsyncToken
        :param response_status_code: The status code returned from a request, as follows:   - 202 — Initialized the Listener status request and is in progress (QUERY response).  - 200 — Listener status request is complete (GET response).
        :type response_status_code: int
        """
        self.async_token = self._define_object(async_token, AsyncToken)
        self.response_status_code = response_status_code
        self._kwargs = kwargs
